@@ -28,7 +28,6 @@
 #include "electron.h"
 #include "text.h"
 #include "plugin.h"
-#include "filechooser.h"
 #include "mendeleiev.h"
 #include "mesomer.h"
 #include "mesomery.h"
@@ -38,7 +37,10 @@
 #include "reaction-step.h"
 #include "reaction-arrow.h"
 #include "tools.h"
+#include "window.h"
 #include "zoomdlg.h"
+#include <gcu/filechooser.h>
+#include <goffice/utils/go-file.h>
 #include <bonobo/bonobo-ui-util.h>
 #include <sys/stat.h>
 #include <gconf/gconf-client.h>
@@ -49,6 +51,8 @@
 #include <openbabel/mol.h>
 #include <openbabel/obconversion.h>
 
+bool new_ui;
+
 // Objects creation static methods
 static Object* CreateMolecule()
 {
@@ -110,60 +114,15 @@ map<string, string> gcpApplication::m_SupportedPixbufFormats;
 #undef PACKAGE
 #define PACKAGE "gchempaint-unstable" 
 
-gcpApplication::gcpApplication (): Application (PACKAGE, DATADIR)
+gcpApplication::gcpApplication (): Application ("GChemPaint", DATADIR, PACKAGE, "gchempaint")
 {
 	Apps.push_back(this);
 	m_CurZ = 6;
 	m_pActiveDoc = NULL;
 	m_pActiveTool = NULL;
 	m_pNode = NULL;
+	m_NumWindow = 1;
 	FontName = g_strdup("Bitstream Vera Serif 12");
-	CurDir = g_get_current_dir ();
-	GConfClient* cli = gconf_client_get_default ();
-	if (cli) {
-		const char *value;
-/*		GConfEntry* entry = gconf_client_get_entry (cli, "/desktop/gnome/applications/help_viewer/exec", NULL, true, NULL);
-		if (entry) {
-			value = gconf_value_get_string (gconf_entry_get_value (entry));
-			if (value) HelpBrowser = value;
-		}*/
-		GConfEntry* entry = gconf_client_get_entry (cli, "/desktop/gnome/applications/browser/exec", NULL, true, NULL);
-		if (entry) {
-			value = gconf_value_get_string (gconf_entry_get_value (entry));
-			if (value) WebBrowser = value;
-		}
-		entry = gconf_client_get_entry (cli, "/desktop/gnome/url-handlers/mailto/command", NULL, true, NULL);
-		if (entry) {
-			value = gconf_value_get_string (gconf_entry_get_value (entry));
-			if (value) {
-				MailAgent = value;
-				int i = MailAgent.find (" %s");
-				if (i)
-					MailAgent.erase (i, MailAgent.size ());
-			}
-		}
-	}
-
-	// search help file
-/*	string lang = getenv ("LANG");
-	HelpFilename = string (DATADIR"/gnome/help/") + string (PACKAGE"/") + lang + "/"PACKAGE".xml";
-	struct stat buf;
-	int err = stat(HelpFilename.c_str (), &buf);
-	if (err) {
-		if (lang.size () > 2) {
-			lang.erase (2, lang.size());
-			HelpFilename = string (DATADIR"/gnome/help/"PACKAGE"/") + lang + "/"PACKAGE".xml";
-			err = stat(HelpFilename.c_str (), &buf);
-		}
-		if (err) {
-			HelpFilename = DATADIR"/gnome/help/"PACKAGE"/C/"PACKAGE".xml";
-			err = stat(HelpFilename.c_str (), &buf);
-		}
-	}
-	if (err) {
-		g_warning("Help file not found");
-		HelpFilename = "";
-	}*/
 
 	if (!m_bInit) {
 		// Check for programs
@@ -252,17 +211,27 @@ gcpApplication::gcpApplication (): Application (PACKAGE, DATADIR)
 	g_object_unref (G_OBJECT (IconFactory));
 	XmlDoc = xmlNewDoc ((xmlChar const*) "1.0");
 	visible_windows = 0;
+	m_idle = 0;
+	m_SupportedMimeTypes.push_back ("application/x-gchempaint");
+	m_SupportedMimeTypes.push_back ("chemical/x-chemdraw");
+	m_SupportedMimeTypes.push_back ("chemical/x-cml");
+	m_SupportedMimeTypes.push_back ("chemical/x-mdl-molfile");
+	m_SupportedMimeTypes.push_back ("chemical/x-pdb");
 }
 
 gcpApplication::~gcpApplication()
 {
 	map<string, gcpTool*>::iterator tool = Tools.begin(), endtool = Tools.end();
-	for (; tool!= endtool; tool++) delete (*tool).second;
-	Tools.clear();
-	Apps.remove(this);
+	for (; tool!= endtool; tool++)
+		delete (*tool).second;
+	Tools.clear ();
+	Apps.remove (this);
 	g_free (FontName);
-	if (m_pNode) bonobo_ui_node_free (m_pNode);
-	if (XmlDoc) xmlFreeDoc (XmlDoc);
+	if (m_pNode)
+		bonobo_ui_node_free (m_pNode);
+	if (XmlDoc)
+		xmlFreeDoc (XmlDoc);
+	m_SupportedMimeTypes.clear ();
 }
 
 void gcpApplication::ActivateMenu(const string& menuname, bool activate)
@@ -304,7 +273,7 @@ void gcpApplication::SetStatusText(const char* text)
 
 void gcpApplication::OnSaveAs()
 {
-	new gcpFileChooser (this, true, m_pActiveDoc);
+	FileChooser (this, true, m_SupportedMimeTypes, m_pActiveDoc);
 }
 
 void gcpApplication::AddUI (const char* filename)
@@ -349,9 +318,10 @@ void gcpApplication::AddUI (const char* filename)
 	}
 }
 
-bool gcpApplication::FileProcess(const gchar* filename, int filetype, bool bSave, GtkWindow *window, gcpDocument *pDoc)
+bool gcpApplication::FileProcess(const gchar* filename, const gchar* mime_type, bool bSave, GtkWindow *window, Document *Doc)
 {
 	const gchar* ext;
+	gcpDocument *pDoc = static_cast<gcpDocument*> (Doc);
 	if (!filename || !strlen(filename) || filename[strlen(filename) - 1] == '/')
 	{
 		GtkWidget* message = gtk_message_dialog_new (window, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
@@ -361,33 +331,33 @@ bool gcpApplication::FileProcess(const gchar* filename, int filetype, bool bSave
 		gtk_widget_destroy (message);
 		return true;
 	}
-	int n = strlen(filename), i = n - 1;
+	if (!mime_type)
+		mime_type = "application/x-gchempaint";
+	int filetype = (mime_type)? 0: gcp_ft_Normal;
+	int n = strlen (filename), i = n - 1;
 	while ((i > 0) && (filename[i] != '.') && (filename[i] != '/')) i--;
 	if (filename[i] == '/') i = 0;
-	ext = NULL;
-	if (!filetype) //All files: search for the extension
-	{
-		if (i > 0)
-		{
-			ext = filename + i;
-			if (!strcmp(ext, ".gchempaint")) filetype = gcp_ft_Normal;
-			else if (!strcmp(ext, ".cml")) filetype = gcp_ft_CML;
-			else if (!strcmp(ext, ".mol")) filetype = gcp_ft_SDF;
-			else if (!strcmp(ext, ".sdf")) filetype = gcp_ft_SDF;
-			else if (!strcmp(ext, ".sd")) filetype = gcp_ft_SDF;
-			else if (!strcmp(ext, ".pdb")) filetype = gcp_ft_PDB;
-			else if (!strcmp(ext, ".ent")) filetype = gcp_ft_PDB;
-			else if (!strcmp(ext, ".chm")) filetype = gcp_ft_ChemDraw;
-			else {
-				GtkWidget* message = gtk_message_dialog_new (window, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
-															_("Sorry, format not supported!"));
-				gtk_window_set_icon_name (GTK_WINDOW (message), "gchempaint");
-				gtk_dialog_run (GTK_DIALOG (message));
-				gtk_widget_destroy (message);
-				return true;
-			}
+	ext = (i > 0)? filename + i: NULL;
+	if (!filetype) {
+		// Check mime type
+		if (!strcmp (mime_type, "application/x-gchempaint"))
+			filetype = gcp_ft_Normal;
+		else if (!strcmp (mime_type, "chemical/x-cml"))
+			filetype = gcp_ft_CML;
+		else if (!strcmp (mime_type, "chemical/x-mdl-molfile"))
+			filetype = gcp_ft_SDF;
+		else if (!strcmp (mime_type, "chemical/x-pdb"))
+			filetype = gcp_ft_PDB;
+		else if (!strcmp (mime_type, "chemical/x-chemdraw"))
+			filetype = gcp_ft_ChemDraw;
+		else {
+			GtkWidget* message = gtk_message_dialog_new (window, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, 
+														_("Sorry, format not supported!"));
+			gtk_window_set_icon_name (GTK_WINDOW (message), "gchempaint");
+			gtk_dialog_run (GTK_DIALOG (message));
+			gtk_widget_destroy (message);
+			return true;
 		}
-		else filetype = gcp_ft_Normal;
 	}
 	if (!ext)
 		switch (filetype)
@@ -658,7 +628,10 @@ void gcpApplication::OpenGcp(const gchar* filename, gcpDocument* pDoc)
 		setlocale(LC_NUMERIC, "C");
 		old_time_locale = g_strdup(setlocale(LC_TIME, NULL));
 		setlocale(LC_TIME, "C");
-		if (!pDoc) pDoc = new gcpDocument(this, true);
+		if (!pDoc) {
+			OnFileNew ();
+			pDoc = m_pActiveDoc;
+		}
 		pDoc->SetFileName(filename, ".gchempaint");
 		bool result = pDoc->Load(xml->children);
 		setlocale(LC_NUMERIC, old_num_locale);
@@ -667,10 +640,9 @@ void gcpApplication::OpenGcp(const gchar* filename, gcpDocument* pDoc)
 		g_free(old_time_locale);
 		if (!result)
 		{
-			delete pDoc;
+			OnFileClose ();
 			throw (int) 4;
 		}
-		AddDocument(pDoc);
 		if (!(info->permissions & (GNOME_VFS_PERM_USER_WRITE | GNOME_VFS_PERM_GROUP_WRITE)))
 			pDoc->SetReadOnly (true);
 		gnome_vfs_file_info_unref (info);
@@ -737,47 +709,6 @@ void gcpApplication::InitTools()
 			(*i).second->Activate ((*i).first == "Select");
 }
 
-void gcpApplication::OnBug ()
-{
-	if (!WebBrowser.size ())
-		return;
-	char *argv[3] = {NULL, PACKAGE_BUGREPORT, NULL};
-	argv[0] = (char*) WebBrowser.c_str();
-	g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
-		NULL, NULL, NULL, NULL);
-}
-
-void gcpApplication::ShowURI (string& uri)
-{
-	if (!WebBrowser.size ())
-		return;
-	char *argv[3] = {NULL, NULL, NULL};
-	argv[0] = (char*) WebBrowser.c_str();
-	argv[1] = (char*) uri.c_str ();
-	g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
-		NULL, NULL, NULL, NULL);
-}
-
-void gcpApplication::OnWeb ()
-{
-	if (!WebBrowser.size ())
-		return;
-	char *argv[3] = {NULL, "http://www.nongnu.org/gchempaint", NULL};
-	argv[0] = (char*) WebBrowser.c_str();
-	g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
-		NULL, NULL, NULL, NULL);
-}
-
-void gcpApplication::OnMail ()
-{
-	if (!MailAgent.size ())
-		return;
-	char *argv[3] = {NULL, "mailto:gchempaint-main@nongnu.org", NULL};
-	argv[0] = (char*) MailAgent.c_str();
-	g_spawn_async (NULL, argv, NULL, G_SPAWN_SEARCH_PATH,
-		NULL, NULL, NULL, NULL);
-}
-
 void gcpApplication::OnShowElements ()
 {
 	gcpMendeleievDlg* dlg = (gcpMendeleievDlg*) GetDialog("Mendeleiev");
@@ -854,13 +785,6 @@ void gcpApplication::OnSaveAsImage ()
 	return;	
 }
 
-void gcpApplication::SetCurDir (char const* dir)
-{
-	if (CurDir)
-		g_free (CurDir);
-	CurDir = g_strdup (dir);
-}
-
 void gcpApplication::Zoom (double zoom)
 {
 	gcpView *pView = m_pActiveDoc->GetView ();
@@ -886,11 +810,11 @@ void gcpApplication::OnToolChanged (GtkAction *current)
 	if (m_pActiveTool)
 		m_pActiveTool->Activate(false);
 	m_pActiveTool = Tools[gtk_action_get_name (current)];
-	if (m_pActiveTool)
-		m_pActiveTool->Activate(true);
 	gcpTools *Tools = dynamic_cast<gcpTools*> (Dialogs["Tools"]);
 	if (Tools)
 		Tools->OnSelectTool (m_pActiveTool);		
+	if (m_pActiveTool)
+		m_pActiveTool->Activate(true);
 }
 
 void gcpApplication::BuildTools ()
@@ -988,18 +912,42 @@ void gcpApplication::AddWindow (gcpWindow *window)
 void gcpApplication::DeleteWindow (gcpWindow *window)
 {
 	m_Windows.erase (window);
-	NotifyIconification (true);
+	ShowTools (false);
 }
 
 void gcpApplication::NotifyIconification (bool iconified)
 {
 	if (iconified) {
-		visible_windows--;
-		if (visible_windows == 0)
-			ShowTools (false);
-	} else {
-		if (visible_windows == 0)
-			ShowTools (true);
-		visible_windows++;
+		ShowTools (false);
 	}
 }
+
+static bool on_wait_focus (gcpApplication *App)
+{
+	App->CheckFocus ();
+	return false;
+}
+
+void gcpApplication::CheckFocus ()
+{
+	ShowTools (focused || gtk_grab_get_current ());
+	m_idle = 0;
+}
+
+void gcpApplication::NotifyFocus (bool has_focus, gcpWindow *window)
+{
+	if (window) {
+		m_pActiveWin = window;
+		m_pActiveDoc = window->GetDocument ();
+		m_pActiveTool->Activate ();
+	}
+	focused = has_focus;
+	if (!m_idle)
+		g_idle_add ((GSourceFunc) on_wait_focus, this);
+}
+
+void gcpApplication::CloseAll ()
+{
+	while (!m_Windows.empty ())
+		(*m_Windows.begin ())->Destroy ();
+}
