@@ -4,7 +4,7 @@
  * GChemPaint library
  * atom.cc
  *
- * Copyright (C) 2001-2007 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -22,35 +22,23 @@
  * USA
  */
 
-#ifndef HAS_OPENBABEL_2
-#	include <mol.h>
-#	undef PACKAGE
-#	undef PACKAGE_BUGREPORT
-#	undef PACKAGE_NAME
-#	undef PACKAGE_STRING
-#	undef PACKAGE_TARNAME
-#	undef PACKAGE_VERSION
-#	undef VERSION
-#endif
-
 #include "gchempaint-config.h"
 #include "atom.h"
 #include "electron.h"
 #include "bond.h"
 #include "molecule.h"
 #include "settings.h"
+#include "theme.h"
 #include "view.h"
 #include "document.h"
 #include "libgcpcanvas/gcp-canvas-group.h"
 #include "libgcpcanvas/gcp-canvas-rect-ellipse.h"
 #include "libgcpcanvas/gcp-canvas-bpath.h"
 #include "libgcpcanvas/gcp-canvas-rich-text.h"
-#include "libgcpcanvas/gcp-canvas-text.h"
+#include "libgcpcanvas/gcp-canvas-pango.h"
+#include <math.h>
 #include <gcu/element.h>
-#include <cmath>
-#ifdef HAS_OPENBABEL_2
-#	include <openbabel/mol.h>
-#endif
+#include <openbabel/mol.h>
 
 using namespace gcu;
 
@@ -60,7 +48,6 @@ extern GtkTextTagTable* TextTagTable;
 
 gcpAtom::gcpAtom(): Atom()
 {
-	m_buf = gtk_text_buffer_new(TextTagTable);
 	m_Valence = -1; //unspecified
 	m_nlp = 0;
 	m_nH = 0;
@@ -75,6 +62,8 @@ gcpAtom::gcpAtom(): Atom()
 	m_ChargeAngle = 0.;
 	m_ChargeDist = 0.;
 	m_ChargeAutoPos = true;
+	m_Layout = m_ChargeLayout = NULL;
+	m_DrawCircle = false;
 }
 
 gcpAtom::~gcpAtom()
@@ -90,11 +79,14 @@ gcpAtom::~gcpAtom()
 		delete electron;
 		electron = GetNextChild (i);
 	}
+	if (m_Layout)
+		g_object_unref (G_OBJECT (m_Layout));
+	if (m_ChargeLayout)
+		g_object_unref (G_OBJECT (m_ChargeLayout));
 }
 
 gcpAtom::gcpAtom(int Z, double x, double y, double z): Atom(Z, x, y, z)
 {
-	m_buf = gtk_text_buffer_new(TextTagTable);
 	m_ChargeAuto = false;
 	m_HPos = GetBestSide();
 	m_nlp = 0;
@@ -108,11 +100,12 @@ gcpAtom::gcpAtom(int Z, double x, double y, double z): Atom(Z, x, y, z)
 	m_ChargeAngle = 0.;
 	m_ChargeDist = 0.;
 	m_ChargeAutoPos = true;
+	m_Layout = m_ChargeLayout = NULL;
+	m_DrawCircle = false;
 }
 
 gcpAtom::gcpAtom(OBAtom* atom, double zoom): Atom()
 {
-	m_buf = gtk_text_buffer_new(TextTagTable);
 	m_x = atom->GetX() * zoom;
 	m_y = - atom->GetY() * zoom;
 	m_z = atom->GetZ() * zoom;
@@ -121,7 +114,6 @@ gcpAtom::gcpAtom(OBAtom* atom, double zoom): Atom()
 	gchar* Id = g_strdup_printf("a%d", atom->GetIdx());
 	SetId(Id);
 	g_free(Id);
-//	m_Valence = atom->GetValence();
 	m_HPos = true;
 	m_ascent = 0;
 	m_CHeight = 0.;
@@ -132,6 +124,8 @@ gcpAtom::gcpAtom(OBAtom* atom, double zoom): Atom()
 	m_ChargeAngle = 0.;
 	m_ChargeDist = 0.;
 	m_ChargeAutoPos = true;
+	m_Layout = m_ChargeLayout = NULL;
+	m_DrawCircle = false;
 }
 
 void gcpAtom::SetZ(int Z)
@@ -268,6 +262,27 @@ void gcpAtom::Update ()
 	if (pDoc)
 		m_Changed = pDoc->GetView ()->GetNbWidgets ();
 	m_AvailPosCached = false;
+	if (nbonds && GetZ () == 6) {
+		// update large bonds ends
+		gcpBond *bond;
+		gcpBondType type;
+		bool DrawCircle;
+		map<Atom*, Bond*>::iterator i = m_Bonds.begin(), iend = m_Bonds.end ();
+		int nb = 0;
+		while (i != iend)
+		{
+			bond = dynamic_cast<gcpBond*> ((gcpBond*)(*i).second);
+			type = bond->GetType ();
+			if (type == ForeBondType || (type == UpBondType && bond->GetAtom (1) == this))
+				nb++;
+			i++;
+		}
+		DrawCircle = nb > 1;
+		if (DrawCircle != m_DrawCircle) {
+			m_DrawCircle = DrawCircle;
+			m_Changed = true;
+		}
+	}
 }
 	
 bool gcpAtom::IsInCycle(gcpCycle* pCycle)
@@ -278,151 +293,118 @@ bool gcpAtom::IsInCycle(gcpCycle* pCycle)
 	return false;
 }
 
-void gcpAtom::Add(GtkWidget* w)
+void gcpAtom::Add (GtkWidget* w)
 {
-	if (!w) return;
-	if (m_Changed > 0) m_Changed--;
-	gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(w), "data");
+	if (!w)
+		return;
+	if (m_Changed > 0)
+		m_Changed--;
+	gcpWidgetData* pData = (gcpWidgetData*) g_object_get_data (G_OBJECT (w), "data");
 	gcpView* pView = pData->View;
-	if (m_ascent <= 0)
-	{
-		PangoContext* pc = pData->View->GetPangoContext();
-		PangoLayout *pl = pango_layout_new(pc);
-		pango_layout_set_text(pl, "l", 1);
-		PangoLayoutIter* iter = pango_layout_get_iter(pl);
-		m_ascent = pango_layout_iter_get_baseline(iter) / PANGO_SCALE;
-		pango_layout_iter_free(iter);
-		g_object_unref(pl);
+	gcpTheme *pTheme = pView->GetDoc ()->GetTheme ();
+	if (m_Layout == NULL) {
+		PangoContext* pc = pView->GetPangoContext();
+		m_Layout = pango_layout_new (pc);
+		pango_layout_set_font_description (m_Layout, pView->GetPangoFontDesc ());
+		pango_layout_set_text (m_Layout, "l", 1);
+		PangoLayoutIter* iter = pango_layout_get_iter (m_Layout);
+		m_ascent = pango_layout_iter_get_baseline (iter) / PANGO_SCALE;
+		pango_layout_iter_free (iter);
 	}
-	if (m_CHeight == 0.)
-	{
-		PangoContext* pc = pData->View->GetPangoContext();
-		PangoLayout *pl = pango_layout_new(pc);
-		pango_layout_set_text(pl, "C", 1);
+	if (m_CHeight == 0.) {
+		pango_layout_set_text (m_Layout, "C", 1);
 		PangoRectangle rect;
-		pango_layout_get_extents(pl, &rect, NULL);
-		m_CHeight =  double(rect.height / PANGO_SCALE) / 2.0;
-		g_object_unref(G_OBJECT(pl));
-	}
-	const gchar* FontName = pView->GetSmallFontName();
-	GtkTextTag* tag = gtk_text_tag_table_lookup(TextTagTable, FontName);
-	if (!tag)
-	{
-		tag = gtk_text_tag_new(FontName);
-		PangoFontDescription* pfd = pView->GetPangoSmallFontDesc();
-		g_object_set(G_OBJECT(tag),
-							"family", pango_font_description_get_family(pfd),
-							"size", pango_font_description_get_size(pfd),
-							NULL);
-		gtk_text_tag_table_add(TextTagTable, tag);
-		g_object_unref((GObject*)tag);
-	}
-	FontName = pView->GetFontName();
-	tag = gtk_text_tag_table_lookup(TextTagTable, FontName);
-	if (!tag)
-	{
-		tag = gtk_text_tag_new(FontName);
-		PangoFontDescription* pfd = pView->GetPangoFontDesc();
-		g_object_set(G_OBJECT(tag),
-							"family", pango_font_description_get_family(pfd),
-							"size", pango_font_description_get_size(pfd),
-							NULL);
-		gtk_text_tag_table_add(TextTagTable, tag);
-		g_object_unref((GObject*)tag);
+		pango_layout_get_extents (m_Layout, &rect, NULL);
+		m_CHeight =  double (rect.height / PANGO_SCALE) / 2.0;
 	}
 	double x, y, xc = 0., yc;
-	m_width =  m_height = 2.0 * pData->Padding;
-	GetCoords(&x, &y);
-	x *= pData->ZoomFactor;
-	y *= pData->ZoomFactor;
-	double dFontHeight = pData->View->GetFontHeight();
+	m_width =  m_height = 2.0 * pTheme->GetPadding ();
+	GetCoords (&x, &y);
+	x *= pTheme->GetZoomFactor ();
+	y *= pTheme->GetZoomFactor ();
 	GnomeCanvasItem *item;
 	GnomeCanvasGroup *group, *chgp;
-	int index = 0;
-	char *hs;
-	PangoContext* pc = pData->View->GetPangoContext();
-	group = GNOME_CANVAS_GROUP(gnome_canvas_item_new(pData->Group, gnome_canvas_group_ext_get_type(), NULL));
-	g_signal_connect(G_OBJECT(group), "event", G_CALLBACK(on_event), w);
-	g_object_set_data(G_OBJECT(group), "object", this);
-	if ((GetZ() != 6) || (GetBondsNumber() == 0))
-	{
-		GtkTextIter start, end;
-		int sw, hw = 0, nw = 0;
-		gtk_text_buffer_get_bounds(m_buf, &start, &end);
-		gtk_text_buffer_delete(m_buf, &start, &end);
-		const gchar* symbol = GetSymbol();
-		gtk_text_buffer_insert_with_tags_by_name(m_buf, &start, symbol, strlen(symbol),pView->GetFontName(), NULL);
-		PangoLayout *pl = pango_layout_new(pc);
-		pango_layout_set_text(pl, symbol, strlen(symbol));
+	group = GNOME_CANVAS_GROUP (gnome_canvas_item_new (pData->Group, gnome_canvas_group_ext_get_type (), NULL));
+	g_signal_connect (G_OBJECT (group), "event", G_CALLBACK (on_event), w);
+	g_object_set_data (G_OBJECT (group), "object", this);
+	if ((GetZ () != 6) || (GetBondsNumber () == 0)) {
+		int sw, sp;
+		const gchar* symbol = GetSymbol (), *text;
+		sw = strlen (symbol);
+		int n = GetAttachedHydrogens ();
+		if (n > 0) {
+			if (n > 1) {
+				gchar const *nb =  g_strdup_printf ("%d", n);
+				int np, nw = strlen (nb);
+				if (m_HPos) {
+					text = g_strconcat (symbol, "H", nb, NULL);
+					np = sw + 1;
+					sp = 0;
+				} else {
+					text = g_strconcat ("H", nb, symbol, NULL);
+					np = 1;
+					sp = np + nw;
+				}
+				pango_layout_set_text (m_Layout, text, -1);
+				PangoAttrList *pal = pango_attr_list_new ();
+				PangoAttribute *attr = pango_attr_font_desc_new (pView->GetPangoSmallFontDesc());
+				attr->start_index = np;
+				attr->end_index = np + nw;
+				pango_attr_list_insert (pal, attr);
+				attr = pango_attr_rise_new (-2 * PANGO_SCALE);
+				attr->start_index = np;
+				attr->end_index = np + nw;
+				pango_attr_list_insert (pal, attr);
+				pango_layout_set_attributes (m_Layout, pal);
+				pango_attr_list_unref (pal);
+			} else {
+				if (m_HPos) {
+					text = g_strconcat (symbol, "H", NULL);
+					sp = 0;
+				} else {
+					text = g_strconcat ("H", symbol, NULL);
+					sp = 1;
+				}
+				pango_layout_set_text (m_Layout, text, -1);
+			}
+		} else {
+			text = g_strdup (symbol);
+			sp = 0;
+			pango_layout_set_text (m_Layout, text, -1);
+		}
 		PangoRectangle rect;
-		pango_layout_get_extents(pl, NULL, &rect);
-		m_width = sw = rect.width / PANGO_SCALE;
-		m_height = rect.height / PANGO_SCALE;
-		m_text_height = dFontHeight;
-		int n = GetAttachedHydrogens();
-		if (n > 0)
-		{
-			pango_layout_set_text(pl, "H", 1);
-			PangoRectangle rect;
-			pango_layout_get_extents(pl, NULL, &rect);
-			hw = rect.width / PANGO_SCALE;
-			index = (m_HPos)? strlen(symbol): 0;
-			gtk_text_buffer_get_iter_at_offset(m_buf, &start, index);
-			gtk_text_buffer_insert_with_tags_by_name(m_buf, &start, "H", 1,pView->GetFontName(), NULL);
-		}
-		g_object_unref(pl);
-		if (n > 1)
-		{
-			pango_context_set_font_description(pc, pData->View->GetPangoSmallFontDesc());
-			pl = pango_layout_new(pc);
-			hs = g_strdup_printf("%d", n);
-			pango_layout_set_text(pl, hs, strlen(hs));
-			pango_layout_get_extents(pl, NULL, &rect);
-			nw = rect.width / PANGO_SCALE;
-			g_object_unref(pl);
-			pango_context_set_font_description(pc, pData->View->GetPangoFontDesc());
-			gtk_text_buffer_get_iter_at_offset(m_buf, &start, index + 1);
-			gtk_text_buffer_insert_with_tags_by_name(m_buf, &start, hs, strlen(hs),pView->GetSmallFontName(), "subscript", NULL);
-			gtk_text_buffer_get_iter_at_offset(m_buf, &start, index + 1);
-			gtk_text_buffer_get_iter_at_offset(m_buf, &end, index + 1 + strlen (hs));
-			gtk_text_buffer_remove_tag_by_name(m_buf, pView->GetFontName(), &start, &end);
-			m_text_height += 2;
-		}
-		m_length = sw + hw + nw;
-		m_lbearing = (m_HPos)? sw / 2: sw / 2 + hw + nw;
+		pango_layout_get_extents (m_Layout, NULL, &rect);
+		m_length = double (rect.width / PANGO_SCALE);
+		m_text_height = m_height = rect.height / PANGO_SCALE;
+		pango_layout_index_to_pos (m_Layout, sp, &rect);
+		int st = rect.x / PANGO_SCALE;
+		pango_layout_index_to_pos (m_Layout, sp + sw, &rect);
+		m_lbearing = (st + rect.x / PANGO_SCALE) / 2.;
 
-		item = gnome_canvas_item_new(
+		item = gnome_canvas_item_new (
 							group,
-							gnome_canvas_rect_ext_get_type(),
-							"x1", x - m_lbearing - pData->Padding,
-							"y1", y  - m_ascent + m_CHeight - pData->Padding,
-							"x2", x - m_lbearing + m_length + pData->Padding,
-							"y2", y  - m_ascent + m_CHeight + m_height + pData->Padding,
-							"fill_color", (pData->IsSelected(this))? SelectColor: "white",
+							gnome_canvas_rect_ext_get_type (),
+							"x1", x - m_lbearing - pTheme->GetPadding (),
+							"y1", y  - m_ascent + m_CHeight - pTheme->GetPadding (),
+							"x2", x - m_lbearing + m_length + pTheme->GetPadding (),
+							"y2", y  - m_ascent + m_CHeight + m_height + pTheme->GetPadding (),
+							"fill_color", (pData->IsSelected (this))? SelectColor: "white",
 							NULL);
-		g_object_set_data(G_OBJECT(group), "rect", item);
-		g_signal_connect(G_OBJECT(item), "event", G_CALLBACK(on_event), w);
-		g_object_set_data(G_OBJECT(item), "object", this);
+		g_object_set_data (G_OBJECT (group), "rect", item);
+		g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);
+		g_object_set_data (G_OBJECT (item), "object", this);
 		
 		item = gnome_canvas_item_new (
 							group,
-							gnome_canvas_rich_text_ext_get_type(),
+							gnome_canvas_pango_get_type (),
 							"x", x - m_lbearing,
 							"y", y - m_ascent + m_CHeight,
-							"width", m_length,
-							"height", m_height,
-							"grow_height", false,
-							"editable", false,
-							"cursor_visible", false,
-							"text_direction", GTK_TEXT_DIR_LTR,
-							NULL);
-		gnome_canvas_rich_text_ext_set_buffer(GNOME_CANVAS_RICH_TEXT_EXT(item), m_buf);
-		g_object_set_data(G_OBJECT(group), "symbol", item);
-		g_object_set_data(G_OBJECT(item), "object", this);
-		g_signal_connect(G_OBJECT(item), "event", G_CALLBACK(on_event), w);
-	}
-	else
-	{
+							"layout", m_Layout,
+						NULL);
+		g_object_set_data (G_OBJECT (group), "symbol", item);
+		g_object_set_data (G_OBJECT (item), "object", this);
+		g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), w);
+	} else {
 		item = gnome_canvas_item_new(
 								group,
 								gnome_canvas_rect_ext_get_type(),
@@ -438,10 +420,25 @@ void gcpAtom::Add(GtkWidget* w)
 		gnome_canvas_item_raise(GNOME_CANVAS_ITEM(group), 1);
 		g_signal_connect(G_OBJECT(item), "event", G_CALLBACK(on_event), w);
 		g_object_set_data(G_OBJECT(item), "object", this);
+		if (m_DrawCircle) {
+			double dx = pTheme->GetStereoBondWidth () / 2.;
+			item = gnome_canvas_item_new(
+									group,
+									gnome_canvas_ellipse_ext_get_type(),
+									"x1", x - dx,
+									"y1", y - dx,
+									"x2", x + dx,
+									"y2", y + dx,
+									"fill_color",  (pData->IsSelected(this))? SelectColor: Color,
+									NULL);
+			g_object_set_data(G_OBJECT(group), "bullet", item);
+			g_signal_connect(G_OBJECT(item), "event", G_CALLBACK(on_event), w);
+			g_object_set_data(G_OBJECT(item), "object", this);
+		}
 	}
 	pData->Items[this] = group;
-	m_width /= pData->ZoomFactor;
-	m_height /= pData->ZoomFactor;
+	m_width /= pTheme->GetZoomFactor ();
+	m_height /= pTheme->GetZoomFactor ();
 	/* add charge */
 	int charge = GetCharge ();
 	if (charge) {
@@ -451,46 +448,47 @@ void gcpAtom::Add(GtkWidget* w)
 			x = m_x + m_ChargeDist * cos (m_ChargeAngle);
 			y = m_y - m_ChargeDist * sin (m_ChargeAngle);
 		}
-		x *= pData->ZoomFactor;
-		y *= pData->ZoomFactor;
+		x *= pTheme->GetZoomFactor ();
+		y *= pTheme->GetZoomFactor ();
 		char *fig = NULL;
 		if 	(abs (charge) > 1) {
 			fig = g_strdup_printf ("%d", abs (charge));
-			PangoContext* pc = pData->View->GetPangoContext();
 			PangoRectangle rect;
-			pango_context_set_font_description (pc, pData->View->GetPangoSmallFontDesc ());
-			PangoLayout* pl = pango_layout_new (pc);
-			pango_layout_set_text (pl, fig, -1);
-			pango_layout_get_extents (pl, NULL, &rect);
+			if (!m_ChargeLayout) {
+				PangoContext* pc = pData->View->GetPangoContext();
+				m_ChargeLayout = pango_layout_new (pc);
+				pango_layout_set_font_description (m_ChargeLayout, pData->View->GetPangoSmallFontDesc ());
+			}
+			pango_layout_set_text (m_ChargeLayout, fig, -1);
+			pango_layout_get_extents (m_ChargeLayout, NULL, &rect);
 			m_ChargeWidth = rect.width / PANGO_SCALE;
-			pango_context_set_font_description (pc, pData->View->GetPangoFontDesc ());
-			m_ChargeTWidth = m_ChargeWidth + 1. + ChargeSignSize;
+			m_ChargeTWidth = m_ChargeWidth + 1. + pTheme->GetChargeSignSize ();
 		} else {
 			m_ChargeWidth = 0.;
-			m_ChargeTWidth = ChargeSignSize;
+			m_ChargeTWidth = pTheme->GetChargeSignSize ();
 		}
 		switch (align) {
 		case -2:
-			xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
-			y += ChargeSignHeight / 2.;
+			xc = x + m_ChargeTWidth / 2. - pTheme->GetChargeSignSize ();
+			y += pTheme->GetChargeSignHeight () / 2.;
 			break;
 		case -1:
-			xc = x - ChargeSignSize -1.;
+			xc = x - pTheme->GetChargeSignSize () -1.;
 			break;
 		case 0:
 		case -3:
-			xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
+			xc = x + m_ChargeTWidth / 2. - pTheme->GetChargeSignSize ();
 			break;
 		case 1:
 			xc = x + m_ChargeWidth + 1.;
 			break;
 		case 2:
-			xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
-			y -= ChargeSignHeight / 2.;
+			xc = x + m_ChargeTWidth / 2. - pTheme->GetChargeSignSize ();
+			y -= pTheme->GetChargeSignHeight () / 2.;
 			break;
 		}
 		x = xc - 1.;
-		yc = y - ChargeSignHeight / 2. + ChargeYAlign - ChargeSignSize + 1.;
+		yc = y - pTheme->GetChargeSignHeight () / 2. + pTheme->GetChargeYAlign () - pTheme->GetChargeSignSize () + 1.;
 		chgp = (GnomeCanvasGroup*) gnome_canvas_item_new (
 					group,
 					gnome_canvas_group_ext_get_type(),
@@ -499,10 +497,9 @@ void gcpAtom::Add(GtkWidget* w)
 		if (fig) {
 			item = gnome_canvas_item_new(
 						chgp,
-						gnome_canvas_text_ext_get_type(),
+						gnome_canvas_pango_get_type(),
+						"layout", m_ChargeLayout,
 						"fill_color", (pData->IsSelected(this))? SelectColor: Color,
-						"font-desc", pData->View->GetPangoSmallFontDesc (),
-						"markup", fig,
 						"anchor", GTK_ANCHOR_EAST,
 						"x", x,
 						"y", y,
@@ -515,8 +512,8 @@ void gcpAtom::Add(GtkWidget* w)
 					gnome_canvas_ellipse_ext_get_type (),
 					"x1", xc,
 					"y1", yc,
-					"x2", xc + ChargeSignSize,
-					"y2", yc + ChargeSignSize,
+					"x2", xc + pTheme->GetChargeSignSize (),
+					"y2", yc + pTheme->GetChargeSignSize (),
 					"outline_color", (pData->IsSelected(this))? SelectColor: Color,
 					"width_units", 0.5,
 					NULL
@@ -526,14 +523,14 @@ void gcpAtom::Add(GtkWidget* w)
 		path[0].code = ART_MOVETO_OPEN;
 		path[0].x3 = xc + 1.;
 		path[1].code = ART_LINETO;
-		path[1].x3 = xc + ChargeSignSize - 1.;
-		path[0].y3 = path[1].y3 = yc + ChargeSignSize / 2.;
+		path[1].x3 = xc + pTheme->GetChargeSignSize () - 1.;
+		path[0].y3 = path[1].y3 = yc + pTheme->GetChargeSignSize () / 2.;
 		if (charge > 0) {
 			path[2].code = ART_MOVETO_OPEN;
 			path[2].y3 = yc + 1.;
 			path[3].code = ART_LINETO;
-			path[3].y3 = yc + ChargeSignSize - 1.;
-			path[2].x3 = path[3].x3 = xc + ChargeSignSize / 2.;
+			path[3].y3 = yc + pTheme->GetChargeSignSize () - 1.;
+			path[2].x3 = path[3].x3 = xc + pTheme->GetChargeSignSize () / 2.;
 			path[4].code = ART_END;
 		} else
 			path[2].code = ART_END;
@@ -561,10 +558,11 @@ void gcpAtom::Update(GtkWidget* w)
 {
 	if (!w) return;
 	gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(w), "data");
+	gcpTheme *pTheme = pData->View->GetDoc ()->GetTheme ();
 	double x, y, xc = 0., yc;
 	GetCoords(&x, &y);
-	x *= pData->ZoomFactor;
-	y *= pData->ZoomFactor;
+	x *= pTheme->GetZoomFactor ();
+	y *= pTheme->GetZoomFactor ();
 	GnomeCanvasGroup *group = pData->Items[this];
 	if (m_Changed) {
 		BuildItems (pData);
@@ -575,18 +573,28 @@ void gcpAtom::Update(GtkWidget* w)
 								"y", y - m_ascent + m_CHeight,
 								NULL);
 			g_object_set(G_OBJECT(g_object_get_data(G_OBJECT(group), "rect")),
-								"x1", x - m_lbearing - pData->Padding,
-								"y1", y  - m_ascent + m_CHeight - pData->Padding,
-								"x2", x - m_lbearing + m_length + pData->Padding,
-								"y2", y  - m_ascent + m_CHeight + m_text_height + pData->Padding,
+								"x1", x - m_lbearing - pTheme->GetPadding (),
+								"y1", y  - m_ascent + m_CHeight - pTheme->GetPadding (),
+								"x2", x - m_lbearing + m_length + pTheme->GetPadding (),
+								"y2", y  - m_ascent + m_CHeight + m_text_height + pTheme->GetPadding (),
 								NULL);
-		} else
+		} else {
 			g_object_set(G_OBJECT(g_object_get_data(G_OBJECT(group), "rect")),
 									"x1", x - 3,
 									"y1", y - 3,
 									"x2", x + 3,
 									"y2", y + 3,
 								NULL);
+			if (m_DrawCircle) {
+				double dx = pTheme->GetStereoBondWidth () / 2.;
+				g_object_set(G_OBJECT(g_object_get_data(G_OBJECT(group), "bullet")),
+										"x1", x - dx,
+										"y1", y - dx,
+										"x2", x + dx,
+										"y2", y + dx,
+										NULL);
+			}
+		}
 	}
 	void* item = g_object_get_data (G_OBJECT (group), "charge");
 	int charge = GetCharge ();
@@ -598,29 +606,28 @@ void gcpAtom::Update(GtkWidget* w)
 				x = m_x + m_ChargeDist * cos (m_ChargeAngle);
 				y = m_y - m_ChargeDist * sin (m_ChargeAngle);
 			}
-			x *= pData->ZoomFactor;
-			y *= pData->ZoomFactor;
+			x *= pTheme->GetZoomFactor ();
+			y *= pTheme->GetZoomFactor ();
 			GnomeCanvasItem *figure = (GnomeCanvasItem*) g_object_get_data (G_OBJECT (group), "figure");
 			char *fig = NULL;
 			if 	(abs (charge) > 1) {
 				fig = g_strdup_printf ("%d", abs (charge));
-				PangoContext* pc = pData->View->GetPangoContext();
 				PangoRectangle rect;
-				pango_context_set_font_description (pc, pData->View->GetPangoSmallFontDesc ());
-				PangoLayout* pl = pango_layout_new (pc);
-				pango_layout_set_text (pl, fig, -1);
-				pango_layout_get_extents (pl, NULL, &rect);
+				if (!m_ChargeLayout) {
+					PangoContext* pc = pData->View->GetPangoContext();
+					m_ChargeLayout = pango_layout_new (pc);
+					pango_layout_set_font_description (m_ChargeLayout, pData->View->GetPangoSmallFontDesc ());
+				}
+				pango_layout_set_text (m_ChargeLayout, fig, -1);
+				pango_layout_get_extents (m_ChargeLayout, NULL, &rect);
 				m_ChargeWidth = rect.width / PANGO_SCALE;
-				pango_context_set_font_description (pc, pData->View->GetPangoFontDesc ());
 			} else
 				m_ChargeWidth = 0.;
-			m_ChargeTWidth = m_ChargeWidth + 1. + ChargeSignSize;
+			m_ChargeTWidth = m_ChargeWidth + 1. + pTheme->GetChargeSignSize ();
 			if (figure == NULL && fig != NULL) {
 				figure = gnome_canvas_item_new(
 							GNOME_CANVAS_GROUP (item),
-							gnome_canvas_text_ext_get_type(),
-							"fill_color", (pData->IsSelected(this))? SelectColor: Color,
-							"font-desc", pData->View->GetPangoSmallFontDesc (),
+							gnome_canvas_pango_get_type(),
 							"anchor", GTK_ANCHOR_EAST,
 							NULL);
 				g_object_set_data (G_OBJECT (group), "figure", figure);
@@ -630,29 +637,29 @@ void gcpAtom::Update(GtkWidget* w)
 			}
 			switch (align) {
 			case -2:
-				xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
-				y += ChargeSignHeight / 2.;
+				xc = x + m_ChargeTWidth / 2. - pTheme->GetChargeSignSize ();
+				y += pTheme->GetChargeSignHeight () / 2.;
 				break;
 			case -1:
-				xc = x - ChargeSignSize -1.;
+				xc = x - pTheme->GetChargeSignSize () -1.;
 				break;
 			case 0:
 			case -3:
-				xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
+				xc = x + m_ChargeTWidth / 2. - pTheme->GetChargeSignSize ();
 				break;
 			case 1:
 				xc = x + m_ChargeWidth + 1.;
 				break;
 			case 2:
-				xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
-				y -= ChargeSignHeight / 2.;
+				xc = x + m_ChargeTWidth / 2. - pTheme->GetChargeSignSize ();
+				y -= pTheme->GetChargeSignHeight () / 2.;
 				break;
 			}
 			x = xc - 1.;
-			yc = y - ChargeSignHeight / 2. + ChargeYAlign - ChargeSignSize + 1.;
+			yc = y - pTheme->GetChargeSignHeight () / 2. + pTheme->GetChargeYAlign () - pTheme->GetChargeSignSize () + 1.;
 			if (fig) {
 				g_object_set (G_OBJECT (figure),
-							"markup", fig,
+							"layout", m_ChargeLayout,
 							"x", x,
 							"y", y,
 							NULL);
@@ -662,22 +669,22 @@ void gcpAtom::Update(GtkWidget* w)
 			g_object_set (G_OBJECT (item),
 						"x1", xc,
 						"y1", yc,
-						"x2", xc + ChargeSignSize,
-						"y2", yc + ChargeSignSize,
+						"x2", xc + pTheme->GetChargeSignSize (),
+						"y2", yc + pTheme->GetChargeSignSize (),
 						NULL);
 			item = g_object_get_data (G_OBJECT (group), "sign");
 			ArtBpath *path = art_new (ArtBpath, 5);
 			path[0].code = ART_MOVETO_OPEN;
 			path[0].x3 = xc + 1.;
 			path[1].code = ART_LINETO;
-			path[1].x3 = xc + ChargeSignSize - 1.;
-			path[0].y3 = path[1].y3 = yc + ChargeSignSize / 2.;
+			path[1].x3 = xc + pTheme->GetChargeSignSize () - 1.;
+			path[0].y3 = path[1].y3 = yc + pTheme->GetChargeSignSize () / 2.;
 			if (charge > 0) {
 				path[2].code = ART_MOVETO_OPEN;
 				path[2].y3 = yc + 1.;
 				path[3].code = ART_LINETO;
-				path[3].y3 = yc + ChargeSignSize - 1.;
-				path[2].x3 = path[3].x3 = xc + ChargeSignSize / 2.;
+				path[3].y3 = yc + pTheme->GetChargeSignSize () - 1.;
+				path[2].x3 = path[3].x3 = xc + pTheme->GetChargeSignSize () / 2.;
 				path[4].code = ART_END;
 			} else
 				path[2].code = ART_END;
@@ -695,47 +702,48 @@ void gcpAtom::Update(GtkWidget* w)
 				x = (m_x + m_ChargeDist * cos (m_ChargeAngle));
 				y = (m_y - m_ChargeDist * sin (m_ChargeAngle));
 			}
-			x *= pData->ZoomFactor;
-			y *= pData->ZoomFactor;
+			x *= pTheme->GetZoomFactor ();
+			y *= pTheme->GetZoomFactor ();
 
 			char *fig = NULL;
 			if 	(abs (charge) > 1) {
 				fig = g_strdup_printf ("%d", abs (charge));
-				PangoContext* pc = pData->View->GetPangoContext();
+				if (!m_ChargeLayout) {
+					PangoContext* pc = pData->View->GetPangoContext();
+					m_ChargeLayout = pango_layout_new (pc);
+					pango_layout_set_font_description (m_ChargeLayout, pData->View->GetPangoSmallFontDesc ());
+				}
+				pango_layout_set_text (m_ChargeLayout, fig, -1);
 				PangoRectangle rect;
-				pango_context_set_font_description (pc, pData->View->GetPangoSmallFontDesc ());
-				PangoLayout* pl = pango_layout_new (pc);
-				pango_layout_set_text (pl, fig, -1);
-				pango_layout_get_extents (pl, NULL, &rect);
+				pango_layout_get_extents (m_ChargeLayout, NULL, &rect);
 				m_ChargeWidth = rect.width / PANGO_SCALE;
-				pango_context_set_font_description (pc, pData->View->GetPangoFontDesc ());
-				m_ChargeTWidth = m_ChargeWidth + 2 + ChargeSignSize;
+				m_ChargeTWidth = m_ChargeWidth + 2 + pTheme->GetChargeSignSize ();
 			} else {
 				m_ChargeWidth = 0.;
-				m_ChargeTWidth = ChargeSignSize;
+				m_ChargeTWidth = pTheme->GetChargeSignSize ();
 			}
 			switch (align) {
 			case -2:
-				xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
-				y += ChargeSignHeight / 2.;
+				xc = x + m_ChargeTWidth / 2. - pTheme->GetChargeSignSize ();
+				y += pTheme->GetChargeSignHeight () / 2.;
 				break;
 			case -1:
-				xc = x - ChargeSignSize -1.;
+				xc = x - pTheme->GetChargeSignSize () -1.;
 				break;
 			case 0:
 			case -3:
-				xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
+				xc = x + m_ChargeTWidth / 2. - pTheme->GetChargeSignSize ();
 				break;
 			case 1:
 				xc = x + m_ChargeWidth + 1.;
 				break;
 			case 2:
-				xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
-				y -= ChargeSignHeight / 2.;
+				xc = x + m_ChargeTWidth / 2. - pTheme->GetChargeSignSize ();
+				y -= pTheme->GetChargeSignHeight () / 2.;
 				break;
 			}
 			x = xc - 1.;
-			yc = y - ChargeSignHeight / 2. + ChargeYAlign - ChargeSignSize + 1.;
+			yc = y - pTheme->GetChargeSignHeight () / 2. + pTheme->GetChargeYAlign () - pTheme->GetChargeSignSize () + 1.;
 			chgp = (GnomeCanvasGroup*) gnome_canvas_item_new (
 						group,
 						gnome_canvas_group_ext_get_type(),
@@ -744,10 +752,9 @@ void gcpAtom::Update(GtkWidget* w)
 			if (fig) {
 				item = gnome_canvas_item_new(
 							chgp,
-							gnome_canvas_text_ext_get_type(),
+							gnome_canvas_pango_get_type(),
+							"layout", m_ChargeLayout,
 							"fill_color", (pData->IsSelected(this))? SelectColor: Color,
-							"font-desc", pData->View->GetPangoSmallFontDesc (),
-							"markup", fig,
 							"anchor", GTK_ANCHOR_EAST,
 							"x", x,
 							"y", y,
@@ -760,8 +767,8 @@ void gcpAtom::Update(GtkWidget* w)
 						gnome_canvas_ellipse_ext_get_type (),
 						"x1", xc,
 						"y1", yc,
-						"x2", xc + ChargeSignSize,
-						"y2", yc + ChargeSignSize,
+						"x2", xc + pTheme->GetChargeSignSize (),
+						"y2", yc + pTheme->GetChargeSignSize (),
 						"outline_color", (pData->IsSelected(this))? SelectColor: Color,
 						"width_units", .5,
 						NULL
@@ -771,14 +778,14 @@ void gcpAtom::Update(GtkWidget* w)
 			path[0].code = ART_MOVETO_OPEN;
 			path[0].x3 = xc + 1.;
 			path[1].code = ART_LINETO;
-			path[1].x3 = xc + ChargeSignSize - 1.;
-			path[0].y3 = path[1].y3 = yc + ChargeSignSize / 2.;
+			path[1].x3 = xc + pTheme->GetChargeSignSize () - 1.;
+			path[0].y3 = path[1].y3 = yc + pTheme->GetChargeSignSize () / 2.;
 			if (charge > 0) {
 				path[2].code = ART_MOVETO_OPEN;
 				path[2].y3 = yc + 1.;
 				path[3].code = ART_LINETO;
-				path[3].y3 = yc + ChargeSignSize - 1.;
-				path[2].x3 = path[3].x3 = xc + ChargeSignSize / 2.;
+				path[3].y3 = yc + pTheme->GetChargeSignSize () - 1.;
+				path[2].x3 = path[3].x3 = xc + pTheme->GetChargeSignSize () / 2.;
 				path[4].code = ART_END;
 			} else
 				path[2].code = ART_END;
@@ -1110,21 +1117,34 @@ void gcpAtom::SetSelected(GtkWidget* w, int state)
 	gpointer item;
 	gchar *color, *chargecolor;
 	
-	switch (state)
-	{	
-		case SelStateUnselected: color = const_cast<char*> ("white"); chargecolor = const_cast<char*> ("black"); break;
-		case SelStateSelected: chargecolor = color = SelectColor; break;
-		case SelStateUpdating: chargecolor = color = AddColor; break;
-		case SelStateErasing: chargecolor = color = DeleteColor; break;
-		default: color = const_cast<char*> ("white"); chargecolor = const_cast<char*> ("black"); break;
+	switch (state) {	
+	case SelStateUnselected:
+		color = "white";
+		chargecolor = "black";
+		break;
+	case SelStateSelected:
+		chargecolor = color = SelectColor;
+		break;
+	case SelStateUpdating:
+		chargecolor = color = AddColor;
+		break;
+	case SelStateErasing:
+		chargecolor = color = DeleteColor;
+		break;
+	default:
+		color = "white";
+		chargecolor = "black";
+		break;
 	}
 	g_object_set (G_OBJECT(g_object_get_data(G_OBJECT(group), "rect")),
 				"fill_color", color, NULL);
-	if ((item = g_object_get_data(G_OBJECT(group), "figure")))
+	if ((item = g_object_get_data (G_OBJECT (group), "bullet")))
 		g_object_set (item, "fill_color", chargecolor, NULL);
-	if ((item = g_object_get_data(G_OBJECT(group), "circle")))
+	if ((item = g_object_get_data (G_OBJECT (group), "figure")))
+		g_object_set (item, "fill_color", chargecolor, NULL);
+	if ((item = g_object_get_data (G_OBJECT (group), "circle")))
 		g_object_set (item, "outline_color", chargecolor, NULL);
-	if ((item = g_object_get_data(G_OBJECT(group), "sign")))
+	if ((item = g_object_get_data (G_OBJECT (group), "sign")))
 		g_object_set (item, "outline_color", chargecolor, NULL);
 	Object::SetSelected (w, state);
 }
@@ -1157,68 +1177,73 @@ void gcpAtom::BuildItems (gcpWidgetData* pData)
 {
 	GnomeCanvasGroup* group = pData->Items[this];
 	void* item;
-	GtkTextIter start, end;
-	double dFontHeight = pData->View->GetFontHeight();
-	PangoContext* pc = pData->View->GetPangoContext();
+	gcpView* pView = pData->View;
+	gcpTheme *pTheme = pView->GetDoc ()->GetTheme ();
 	double x, y;
-	m_width =  m_height = 2.0 * pData->Padding;
-	GetCoords(&x, &y);
-	x *= pData->ZoomFactor;
-	y *= pData->ZoomFactor;
-	if ((GetZ() != 6) || (GetBondsNumber() == 0))
-	{
-		int sw, hw = 0, nw = 0;
-		int index = 0;
-		char *hs;
-		gtk_text_buffer_get_bounds(m_buf, &start, &end);
-		gtk_text_buffer_delete(m_buf, &start, &end);
-		const gchar* symbol = GetSymbol();
-		gtk_text_buffer_insert_with_tags_by_name(m_buf, &start, symbol, strlen(symbol),pData->View->GetFontName(), NULL);
-		PangoLayout *pl = pango_layout_new(pc);
-		pango_layout_set_text(pl, symbol, strlen(symbol));
+	m_width =  m_height = 2.0 * pTheme->GetPadding ();
+	GetCoords (&x, &y);
+	x *= pTheme->GetZoomFactor ();
+	y *= pTheme->GetZoomFactor ();
+	if ((GetZ() != 6) || (GetBondsNumber() == 0)) {
+		int sw, sp;
+		const gchar* symbol = GetSymbol (), *text;
+		sw = strlen (symbol);
+		int n = GetAttachedHydrogens ();
+		PangoAttrList *pal = pango_attr_list_new ();
+		if (n > 0) {
+			if (n > 1) {
+				gchar const *nb =  g_strdup_printf ("%d", n);
+				int np, nw = strlen (nb);
+				if (m_HPos) {
+					text = g_strconcat (symbol, "H", nb, NULL);
+					np = sw + 1;
+					sp = 0;
+				} else {
+					text = g_strconcat ("H", nb, symbol, NULL);
+					np = 1;
+					sp = np + nw;
+				}
+				pango_layout_set_text (m_Layout, text, -1);
+				PangoAttribute *attr = pango_attr_font_desc_new (pView->GetPangoSmallFontDesc());
+				attr->start_index = np;
+				attr->end_index = np + nw;
+				pango_attr_list_insert (pal, attr);
+				attr = pango_attr_rise_new (-2 * PANGO_SCALE);
+				attr->start_index = np;
+				attr->end_index = np + nw;
+				pango_attr_list_insert (pal, attr);
+			} else {
+				if (m_HPos) {
+					text = g_strconcat (symbol, "H", NULL);
+					sp = 0;
+				} else {
+					text = g_strconcat ("H", symbol, NULL);
+					sp = 1;
+				}
+				pango_layout_set_text (m_Layout, text, -1);
+			}
+			pango_layout_set_attributes (m_Layout, pal);
+			pango_attr_list_unref (pal);
+		} else {
+			text = g_strdup (symbol);
+			sp = 0;
+			pango_layout_set_text (m_Layout, text, -1);
+		}
 		PangoRectangle rect;
-		pango_layout_get_extents(pl, NULL, &rect);
-		m_width = sw = rect.width / PANGO_SCALE;
-		m_height = rect.height / PANGO_SCALE;
-		m_text_height = dFontHeight;
-		int n = GetAttachedHydrogens();
-		if (n > 0)
-		{
-			pango_layout_set_text(pl, "H", 1);
-			PangoRectangle rect;
-			pango_layout_get_extents(pl, NULL, &rect);
-			hw = rect.width / PANGO_SCALE;
-			index = (m_HPos)? strlen(symbol): 0;
-			gtk_text_buffer_get_iter_at_offset(m_buf, &start, index);
-			gtk_text_buffer_insert_with_tags_by_name(m_buf, &start, "H", 1,pData->View->GetFontName(), NULL);
-		}
-		g_object_unref(pl);
-		if (n > 1)
-		{
-			pango_context_set_font_description(pc, pData->View->GetPangoSmallFontDesc());
-			pl = pango_layout_new(pc);
-			hs = g_strdup_printf("%d", n);
-			pango_layout_set_text(pl, hs, strlen(hs));
-			pango_layout_get_extents(pl, NULL, &rect);
-			nw = rect.width / PANGO_SCALE;
-			g_object_unref(pl);
-			pango_context_set_font_description(pc, pData->View->GetPangoFontDesc());
-			gtk_text_buffer_get_iter_at_offset(m_buf, &start, index + 1);
-			gtk_text_buffer_insert_with_tags_by_name(m_buf, &start, hs, strlen(hs),pData->View->GetSmallFontName(), "subscript", NULL);
-			gtk_text_buffer_get_iter_at_offset(m_buf, &start, index + 1);
-			gtk_text_buffer_get_iter_at_offset(m_buf, &end, index + 1 + strlen (hs));
-			gtk_text_buffer_remove_tag_by_name(m_buf, pData->View->GetFontName(), &start, &end);
-			m_text_height += 2;
-		}
-		m_length = sw + hw + nw;
-		m_lbearing = (m_HPos)? sw / 2: sw / 2 + hw + nw;
+		pango_layout_get_extents (m_Layout, NULL, &rect);
+		m_length = double (rect.width / PANGO_SCALE);
+		m_text_height = m_height = rect.height / PANGO_SCALE;
+		pango_layout_index_to_pos (m_Layout, sp, &rect);
+		int st = rect.x / PANGO_SCALE;
+		pango_layout_index_to_pos (m_Layout, sp + sw, &rect);
+		m_lbearing = (st + rect.x / PANGO_SCALE) / 2.;
 
 		item = g_object_get_data (G_OBJECT (group), "rect");
 		g_object_set (item,
-							"x1", x - m_lbearing - pData->Padding,
-							"y1", y  - m_ascent + m_CHeight - pData->Padding,
-							"x2", x - m_lbearing + m_length + pData->Padding,
-							"y2", y  - m_ascent + m_CHeight + m_height + pData->Padding,
+							"x1", x - m_lbearing - pTheme->GetPadding (),
+							"y1", y  - m_ascent + m_CHeight - pTheme->GetPadding (),
+							"x2", x - m_lbearing + m_length + pTheme->GetPadding (),
+							"y2", y  - m_ascent + m_CHeight + m_height + pTheme->GetPadding (),
 							NULL);
 		
 		item = g_object_get_data (G_OBJECT (group), "symbol");
@@ -1226,31 +1251,26 @@ void gcpAtom::BuildItems (gcpWidgetData* pData)
 			g_object_set (item,
 							"x", x - m_lbearing,
 							"y", y - m_ascent + m_CHeight,
-							"width", m_length,
-							"height", m_height,
 							NULL);
 		else {
 			item = gnome_canvas_item_new (
 							group,
-							gnome_canvas_rich_text_ext_get_type(),
+							gnome_canvas_pango_get_type(),
+							"layout", m_Layout,
 							"x", x - m_lbearing,
 							"y", y - m_ascent + m_CHeight,
-							"width", m_length,
-							"height", m_height,
-							"grow_height", false,
-							"editable", false,
-							"cursor_visible", false,
-							"text_direction", GTK_TEXT_DIR_LTR,
 							NULL);
-			gnome_canvas_rich_text_ext_set_buffer(GNOME_CANVAS_RICH_TEXT_EXT(item), m_buf);
-			g_object_set_data(G_OBJECT(group), "symbol", item);
-			g_object_set_data(G_OBJECT(item), "object", this);
-			g_signal_connect(G_OBJECT(item), "event", G_CALLBACK(on_event), pData->Canvas);
-			gnome_canvas_item_raise_to_top(GNOME_CANVAS_ITEM(group));
+			g_object_set_data (G_OBJECT (group), "symbol", item);
+			g_object_set_data (G_OBJECT (item), "object", this);
+			g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), pData->Canvas);
+			gnome_canvas_item_raise_to_top (GNOME_CANVAS_ITEM (group));
 		}
-	}
-	else
-	{
+		item = g_object_get_data (G_OBJECT (group), "bullet");
+		if (item) {
+			gtk_object_destroy (GTK_OBJECT (item));
+			g_object_set_data (G_OBJECT (group), "bullet", NULL);
+		}
+	} else {
 		item = g_object_get_data (G_OBJECT (group), "rect");
 		g_object_set (item,
 							"x1", x - 3,
@@ -1259,16 +1279,38 @@ void gcpAtom::BuildItems (gcpWidgetData* pData)
 							"y2", y + 3,
 							NULL);
 		item = g_object_get_data (G_OBJECT (group), "symbol");
-		if (item){
+		if (item) {
 			gtk_object_destroy (GTK_OBJECT (item));
 			g_object_set_data (G_OBJECT (group), "symbol", NULL);
 		}
+		item = g_object_get_data (G_OBJECT (group), "bullet");
+		if (m_DrawCircle) {
+			if (!item) {
+				double dx = pTheme->GetStereoBondWidth () / 2.;
+				item = gnome_canvas_item_new(
+										group,
+										gnome_canvas_ellipse_ext_get_type (),
+										"x1", x - dx,
+										"y1", y - dx,
+										"x2", x + dx,
+										"y2", y + dx,
+										"fill_color",  (pData->IsSelected (this))? SelectColor: Color,
+										NULL);
+				g_object_set_data (G_OBJECT (group), "bullet", item);
+				g_signal_connect (G_OBJECT (item), "event", G_CALLBACK (on_event), pData->Canvas);
+				g_object_set_data (G_OBJECT (item), "object", this);
+			}
+		} else if (item) {
+			gtk_object_destroy (GTK_OBJECT (item));
+			g_object_set_data (G_OBJECT (group), "bullet", NULL);
+		}
 		m_length = m_text_height = 0;
 		gnome_canvas_item_lower_to_bottom (GNOME_CANVAS_ITEM (group));
 	}
-	m_width /= pData->ZoomFactor;
-	m_height /= pData->ZoomFactor;
-	if (m_Changed > 0) m_Changed--;
+	m_width /= pTheme->GetZoomFactor ();
+	m_height /= pTheme->GetZoomFactor ();
+	if (m_Changed > 0)
+		m_Changed--;
 }
 	
 double gcpAtom::GetYAlign ()
@@ -1390,31 +1432,31 @@ xmlNodePtr gcpAtom::Save (xmlDocPtr xml)
 		if (m_ChargePos) {
 			switch (m_ChargePos) {
 			case CHARGE_NE:
-				buf = const_cast<char*> ("ne");
+				buf = "ne";
 				break;
 			case CHARGE_NW:
-				buf = const_cast<char*> ("nw");
+				buf = "nw";
 				break;
 			case CHARGE_N:
-				buf = const_cast<char*> ("n");
+				buf = "n";
 				break;
 			case CHARGE_SE:
-				buf = const_cast<char*> ("se");
+				buf = "se";
 				break;
 			case CHARGE_SW:
-				buf = const_cast<char*> ("sw");
+				buf = "sw";
 				break;
 			case CHARGE_S:
-				buf = const_cast<char*> ("s");
+				buf = "s";
 				break;
 			case CHARGE_E:
-				buf = const_cast<char*> ("e");
+				buf = "e";
 				break;
 			case CHARGE_W:
-				buf = const_cast<char*> ("w");
+				buf = "w";
 				break;
 			default:
-				buf = const_cast<char*> ("def"); // should not occur
+				buf = "def"; // should not occur
 			}
 			xmlNewProp (node, (xmlChar*) "charge-position", (xmlChar*) buf);
 		} else {
@@ -1558,19 +1600,18 @@ void gcpAtom::Transform2D (Matrix2D& m, double x, double y)
 		electron = GetNextChild (i);
 	}
 	if (GetCharge ()) {
-		m_ChargeAutoPos = false; // should be fixed before 0.6.4
-/*		if (m_ChargeAutoPos) {
+		if (m_ChargeAutoPos) {
 			if (m_ChargePos > 0)
 				NotifyPositionOccupation (m_ChargePos, false);
 			m_ChargePos = 0xff;
 			Update ();
-		} else {*/
+		} else {
 			double xc = cos (m_ChargeAngle), yc = - sin (m_ChargeAngle);
 			m.Transform (xc, yc);
 			m_ChargeAngle = atan2 (- yc, xc);
 			if (m_ChargeAngle < 0)
 				m_ChargeAngle += 2 * M_PI;
 			SetChargePosition (0, FALSE, m_ChargeAngle, m_ChargeDist);
-//		}
+		}
 	}
 }
