@@ -29,6 +29,7 @@
 #include "cycle.h"
 #include "settings.h"
 #include "document.h"
+#include "theme.h"
 #include "view.h"
 #include "widgetdata.h"
 #include "libgcpcanvas/gcp-canvas-group.h"
@@ -118,12 +119,14 @@ bool gcpBond::GetLine2DCoords(unsigned Num, double* x1, double* y1, double* x2,
 	if ((Num == 0) || (Num > m_order)) return false;
 	if (!m_CoordsCalc)
 	{
+		gcpTheme *Theme = dynamic_cast <gcpDocument *> (GetDocument ())->GetTheme ();
 		m_Begin->GetCoords(x1, y1);
 		m_End->GetCoords(x2, y2);
 		double dx = *x2 - *x1, dy = *y2 - *y1;
 		double l = sqrt(square(dx) + square(dy));
-		dx *= (DefaultBondDist / l);
-		dy *= (DefaultBondDist / l);
+		double BondDist = Theme->GetBondDist () / Theme->GetZoomFactor ();
+		dx *= (BondDist / l);
+		dy *= (BondDist / l);
 		if (m_order & 1)
 		{
 			m_coords[0] = *x1;
@@ -164,11 +167,11 @@ bool gcpBond::GetLine2DCoords(unsigned Num, double* x1, double* y1, double* x2,
 			{
 				double sign = sin(a0 - a1) > 0.0 ? 1.0 : -1.0;
 				double tanb = fabs(tan((3.141592653589793238462643 - a0 + a1) / 2)), cosa = cos(a0), sina = sin(a0);
-				m_coords[4] = *x1 + DefaultBondDist * cosa * tanb - dy * sign;
-				m_coords[5] = *y1 + dx * sign - DefaultBondDist * sina * tanb;
+				m_coords[4] = *x1 + BondDist * cosa * tanb - dy * sign;
+				m_coords[5] = *y1 + dx * sign - BondDist * sina * tanb;
 				tanb = fabs(tan((a2 - a0) / 2));
-				m_coords[6] = *x2 - DefaultBondDist * cosa * tanb - dy * sign;
-				m_coords[7] = *y2 + dx * sign + DefaultBondDist * sina * tanb;
+				m_coords[6] = *x2 - BondDist * cosa * tanb - dy * sign;
+				m_coords[7] = *y2 + dx * sign + BondDist * sina * tanb;
 			}
 			else
 			{
@@ -293,6 +296,8 @@ void gcpBond::IncOrder(int n)
 double gcpBond::GetDist(double x, double y)
 {
 	double x1, y1, x2, y2, l, d1, d2;
+	gcpTheme *Theme = dynamic_cast <gcpDocument *> (GetDocument ())->GetTheme ();
+	double BondDist = Theme->GetBondDist () / Theme->GetZoomFactor ();
 	m_Begin->GetCoords(&x1, &y1);
 	m_End->GetCoords(&x2, &y2);
 	d1 = (x2 - x1) * (x1 - x) + (y2 - y1) * (y1 - y);
@@ -304,7 +309,7 @@ double gcpBond::GetDist(double x, double y)
 	x -= x1;
 	y -= y1;
 	l = fabs(x2 * y - y2 * x) / sqrt(square(x2) + square(y2));
-	return (l < DefaultBondDist * (m_order - 1)) ? 0 : l - DefaultBondDist * (m_order - 1);
+	return (l < BondDist * (m_order - 1)) ? 0 : l - BondDist * (m_order - 1);
 }
 
 void gcpBond::AddCycle(gcpCycle* pCycle)
@@ -404,6 +409,7 @@ void gcpBond::Add(GtkWidget* w)
 {
 	if (!w) return;
 	gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(w), "data");
+	gcpTheme *pTheme = pData->View->GetDoc ()->GetTheme ();
 	Atom *pAtom0, *pAtom1;
 	if (!(pAtom0 = GetAtom(0))) return; 
 	if (!(pAtom1 = GetAtom(1))) return;
@@ -432,7 +438,7 @@ void gcpBond::Add(GtkWidget* w)
 					gnome_canvas_bpath_ext_get_type (),
 					"bpath", path,
 					"outline_color", "white",
-					"width_units", pData->BondWidth * 3.,
+					"width_units", pTheme->GetBondWidth () * 3.,
 					NULL):
 				gnome_canvas_item_new (
 					group,
@@ -466,7 +472,7 @@ void gcpBond::Add(GtkWidget* w)
 									gnome_canvas_bpath_ext_get_type(),
 									"bpath", path,
 									"outline_color", (pData->IsSelected(this))? SelectColor: Color,
-									"width_units", pData->BondWidth,
+									"width_units", pTheme->GetBondWidth (),
 									NULL);
 			break;
 		case ForeBondType:
@@ -541,56 +547,57 @@ GnomeCanvasPathDef* gcpBond::BuildPathDef (gcpWidgetData* pData)
 	double x1, y1, x2, y2, x, y, dx, dy, dx1, dy1;
 	int i, n;
 	GnomeCanvasPathDef* path = gnome_canvas_path_def_new ();
+	gcpTheme *pTheme = pData->View->GetDoc ()->GetTheme ();
 	switch (GetType())
 	{
 		case NormalBondType:
 			i = 1;
 			while (GetLine2DCoords(i++, &x1, &y1, &x2, &y2))
 			{
-				gnome_canvas_path_def_moveto (path, x1 * pData->ZoomFactor, y1 * pData->ZoomFactor);
-				gnome_canvas_path_def_lineto (path, x2 * pData->ZoomFactor, y2 * pData->ZoomFactor);
+				gnome_canvas_path_def_moveto (path, x1 * pTheme->GetZoomFactor (), y1 * pTheme->GetZoomFactor ());
+				gnome_canvas_path_def_lineto (path, x2 * pTheme->GetZoomFactor (), y2 * pTheme->GetZoomFactor ());
 			}
 			break;
 		case UpBondType:
 			GetLine2DCoords(1, &x1, &y1, &x2, &y2);
-			gnome_canvas_path_def_moveto (path, x1 * pData->ZoomFactor, y1 * pData->ZoomFactor);
+			gnome_canvas_path_def_moveto (path, x1 * pTheme->GetZoomFactor (), y1 * pTheme->GetZoomFactor ());
 			x = sqrt(square(x2 - x1) + square(y2 - y1));
-			dx = (y1 - y2) / x * pData->StereoBondWidth / 2;
-			dy = (x2 - x1) / x * pData->StereoBondWidth / 2;
-			gnome_canvas_path_def_lineto (path, x2 * pData->ZoomFactor + dx, y2 * pData->ZoomFactor + dy);
-			gnome_canvas_path_def_lineto (path, x2 * pData->ZoomFactor -dx, y2 * pData->ZoomFactor -dy);
+			dx = (y1 - y2) / x * pTheme->GetStereoBondWidth () / 2;
+			dy = (x2 - x1) / x * pTheme->GetStereoBondWidth () / 2;
+			gnome_canvas_path_def_lineto (path, x2 * pTheme->GetZoomFactor () + dx, y2 * pTheme->GetZoomFactor () + dy);
+			gnome_canvas_path_def_lineto (path, x2 * pTheme->GetZoomFactor () -dx, y2 * pTheme->GetZoomFactor () -dy);
 			gnome_canvas_path_def_closepath (path);
 			break;
 		case DownBondType:
 		{
 			double xi[8];
 			GetLine2DCoords(1, &x1, &y1, &x2, &y2);
-			x1 *= pData->ZoomFactor;
-			y1 *= pData->ZoomFactor;
-			x2 *= pData->ZoomFactor;
-			y2 *= pData->ZoomFactor;
+			x1 *= pTheme->GetZoomFactor ();
+			y1 *= pTheme->GetZoomFactor ();
+			x2 *= pTheme->GetZoomFactor ();
+			y2 *= pTheme->GetZoomFactor ();
 			x = sqrt(square(x2 - x1) + square(y2 - y1));
-			n = int(floor(x / (pData->HashDist + pData->HashWidth)));
-			dx1 = (x2 - x1) / x * pData->HashWidth;
-			dy1 = (y2 - y1) / x * pData->HashWidth;
-			dx = (y1 - y2) / x * pData->StereoBondWidth / 2;
-			dy = (x2 - x1) / x * pData->StereoBondWidth / 2;
+			n = int(floor(x / (pTheme->GetHashDist () + pTheme->GetHashWidth ())));
+			dx1 = (x2 - x1) / x * pTheme->GetHashWidth ();
+			dy1 = (y2 - y1) / x * pTheme->GetHashWidth ();
+			dx = (y1 - y2) / x * pTheme->GetStereoBondWidth () / 2;
+			dy = (x2 - x1) / x * pTheme->GetStereoBondWidth () / 2;
 			gnome_canvas_path_def_moveto (path, xi[0] = x1 + dx, xi[1] = y1 + dy);
 			gnome_canvas_path_def_lineto (path, xi[2] = x1 - dx, xi[3] = y1 - dy);
-			dx *= (1 - pData->HashWidth / x);
-			dy *= (1 - pData->HashWidth / x);
+			dx *= (1 - pTheme->GetHashWidth () / x);
+			dy *= (1 - pTheme->GetHashWidth () / x);
 			gnome_canvas_path_def_lineto (path, xi[4] = x1 + dx1 - dx, xi[5] = y1 + dy1 - dy);
 			gnome_canvas_path_def_lineto (path, xi[6] = x1 + dx1 + dx, xi[7] = y1 + dy1 + dy);
 			gnome_canvas_path_def_lineto (path, xi[0], xi[1]);
 			gnome_canvas_path_def_closepath_current (path);
-			dx = (x2 - x1) / x * (pData->HashDist + pData->HashWidth)
-				- (y1 - y2) / x * pData->StereoBondWidth / 2 * (pData->HashDist + pData->HashWidth) / x;
-			dy = (y2 - y1) / x * (pData->HashDist + pData->HashWidth)
-				- (x2 - x1) / x * pData->StereoBondWidth / 2 *  (pData->HashDist + pData->HashWidth) / x;
-			dx1 = (x2 - x1) / x * (pData->HashDist + pData->HashWidth)
-				+ (y1 - y2) / x * pData->StereoBondWidth / 2 *  (pData->HashDist + pData->HashWidth) / x;
-			dy1 = (y2 - y1) / x * (pData->HashDist + pData->HashWidth)
-				+ (x2 - x1) / x * pData->StereoBondWidth / 2 *  (pData->HashDist + pData->HashWidth) / x;
+			dx = (x2 - x1) / x * (pTheme->GetHashDist () + pTheme->GetHashWidth ())
+				- (y1 - y2) / x * pTheme->GetStereoBondWidth () / 2 * (pTheme->GetHashDist () + pTheme->GetHashWidth ()) / x;
+			dy = (y2 - y1) / x * (pTheme->GetHashDist () + pTheme->GetHashWidth ())
+				- (x2 - x1) / x * pTheme->GetStereoBondWidth () / 2 *  (pTheme->GetHashDist () + pTheme->GetHashWidth ()) / x;
+			dx1 = (x2 - x1) / x * (pTheme->GetHashDist () + pTheme->GetHashWidth ())
+				+ (y1 - y2) / x * pTheme->GetStereoBondWidth () / 2 *  (pTheme->GetHashDist () + pTheme->GetHashWidth ()) / x;
+			dy1 = (y2 - y1) / x * (pTheme->GetHashDist () + pTheme->GetHashWidth ())
+				+ (x2 - x1) / x * pTheme->GetStereoBondWidth () / 2 *  (pTheme->GetHashDist () + pTheme->GetHashWidth ()) / x;
 			for (int i = 1; i < n; i++)
 			{
 				gnome_canvas_path_def_moveto (path, xi[0] += dx, xi[1] += dy);
@@ -605,21 +612,21 @@ GnomeCanvasPathDef* gcpBond::BuildPathDef (gcpWidgetData* pData)
 		case ForeBondType:
 			GetLine2DCoords(1, &x1, &y1, &x2, &y2);
 			x = sqrt(square(x2 - x1) + square(y2 - y1));
-			dx = (y1 - y2) / x * pData->StereoBondWidth / 2;
-			dy = (x2 - x1) / x * pData->StereoBondWidth / 2;
-			gnome_canvas_path_def_moveto (path, x1 * pData->ZoomFactor + dx, y1 * pData->ZoomFactor + dy);
-			gnome_canvas_path_def_lineto (path, x2 * pData->ZoomFactor + dx, y2 * pData->ZoomFactor + dy);
-			gnome_canvas_path_def_lineto (path, x2 * pData->ZoomFactor -dx, y2 * pData->ZoomFactor -dy);
-			gnome_canvas_path_def_lineto (path, x1 * pData->ZoomFactor -dx, y1 * pData->ZoomFactor -dy);
+			dx = (y1 - y2) / x * pTheme->GetStereoBondWidth () / 2;
+			dy = (x2 - x1) / x * pTheme->GetStereoBondWidth () / 2;
+			gnome_canvas_path_def_moveto (path, x1 * pTheme->GetZoomFactor () + dx, y1 * pTheme->GetZoomFactor () + dy);
+			gnome_canvas_path_def_lineto (path, x2 * pTheme->GetZoomFactor () + dx, y2 * pTheme->GetZoomFactor () + dy);
+			gnome_canvas_path_def_lineto (path, x2 * pTheme->GetZoomFactor () -dx, y2 * pTheme->GetZoomFactor () -dy);
+			gnome_canvas_path_def_lineto (path, x1 * pTheme->GetZoomFactor () -dx, y1 * pTheme->GetZoomFactor () -dy);
 			gnome_canvas_path_def_closepath (path);
 			break;
 		case UndeterminedBondType:
 		{
 			GetLine2DCoords(1, &x1, &y1, &x2, &y2);
-			x1 *= pData->ZoomFactor;
-			y1 *= pData->ZoomFactor;
-			x2 *= pData->ZoomFactor;
-			y2 *= pData->ZoomFactor;
+			x1 *= pTheme->GetZoomFactor ();
+			y1 *= pTheme->GetZoomFactor ();
+			x2 *= pTheme->GetZoomFactor ();
+			y2 *= pTheme->GetZoomFactor ();
 			gnome_canvas_path_def_moveto(path, x1, y1);
 			double length;
 			length = sqrt(square(x2 - x1) + square(y2 - y1));
@@ -653,6 +660,7 @@ GnomeCanvasPathDef* gcpBond::BuildPathDef (gcpWidgetData* pData)
 
 GnomeCanvasPathDef *gcpBond::BuildCrossingPathDef (gcpWidgetData* pData)
 {
+	gcpTheme *pTheme = pData->View->GetDoc ()->GetTheme ();
 	GnomeCanvasPathDef* path = NULL;
 	double x1, y1, x2, y2, x, dx, dy;
 	int i;
@@ -667,8 +675,8 @@ GnomeCanvasPathDef *gcpBond::BuildCrossingPathDef (gcpWidgetData* pData)
 			x2 -= dx;
 			y1 += dy;
 			y2 -= dy;
-			gnome_canvas_path_def_moveto (path, x1 * pData->ZoomFactor, y1 * pData->ZoomFactor);
-			gnome_canvas_path_def_lineto (path, x2 * pData->ZoomFactor, y2 * pData->ZoomFactor);
+			gnome_canvas_path_def_moveto (path, x1 * pTheme->GetZoomFactor (), y1 * pTheme->GetZoomFactor ());
+			gnome_canvas_path_def_lineto (path, x2 * pTheme->GetZoomFactor (), y2 * pTheme->GetZoomFactor ());
 		}
 		break;
 	case UpBondType:
@@ -685,14 +693,14 @@ GnomeCanvasPathDef *gcpBond::BuildCrossingPathDef (gcpWidgetData* pData)
 		y1 += dy;
 		y2 -= dy;
 		x = sqrt(square(x2 - x1) + square(y2 - y1));
-		dx = (y1 - y2) / x * pData->StereoBondWidth / 2;
-		dy = (x2 - x1) / x * pData->StereoBondWidth / 2;
+		dx = (y1 - y2) / x * pTheme->GetStereoBondWidth () / 2;
+		dy = (x2 - x1) / x * pTheme->GetStereoBondWidth () / 2;
 		dx += (dx > 0.)? 1.: -1.;
 		dy += (dy > 0.)? 1.: -1.;
-		gnome_canvas_path_def_moveto (path, x1 * pData->ZoomFactor + dx, y1 * pData->ZoomFactor + dy);
-		gnome_canvas_path_def_lineto (path, x2 * pData->ZoomFactor + dx, y2 * pData->ZoomFactor + dy);
-		gnome_canvas_path_def_lineto (path, x2 * pData->ZoomFactor -dx, y2 * pData->ZoomFactor -dy);
-		gnome_canvas_path_def_lineto (path, x1 * pData->ZoomFactor -dx, y1 * pData->ZoomFactor -dy);
+		gnome_canvas_path_def_moveto (path, x1 * pTheme->GetZoomFactor () + dx, y1 * pTheme->GetZoomFactor () + dy);
+		gnome_canvas_path_def_lineto (path, x2 * pTheme->GetZoomFactor () + dx, y2 * pTheme->GetZoomFactor () + dy);
+		gnome_canvas_path_def_lineto (path, x2 * pTheme->GetZoomFactor () -dx, y2 * pTheme->GetZoomFactor () -dy);
+		gnome_canvas_path_def_lineto (path, x1 * pTheme->GetZoomFactor () -dx, y1 * pTheme->GetZoomFactor () -dy);
 		gnome_canvas_path_def_closepath (path);
 		break;
 	case UndeterminedBondType:
@@ -705,6 +713,7 @@ void gcpBond::Update(GtkWidget* w)
 {
 	if (!w) return;
 	gcpWidgetData* pData = (gcpWidgetData*) g_object_get_data(G_OBJECT(w), "data");
+	gcpTheme *pTheme = pData->View->GetDoc ()->GetTheme ();
 	bool result = false;
 	if (m_Crossing.size () > 0) {
 		map<gcpBond*, gcpBondCrossing>::iterator i, iend = m_Crossing.end ();
@@ -728,7 +737,7 @@ void gcpBond::Update(GtkWidget* w)
 						gnome_canvas_bpath_ext_get_type (),
 						"bpath", path,
 						"outline_color", "white",
-						"width_units", pData->BondWidth * 3.,
+						"width_units", pTheme->GetBondWidth () * 3.,
 						NULL):
 					gnome_canvas_item_new (
 						group,
