@@ -4,7 +4,7 @@
  * GChemPaint library
  * docprop.h 
  *
- * Copyright (C) 2002-2004 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -25,21 +25,24 @@
 #ifndef GCHEMPAINT_DOC_PROP_H
 #define GCHEMPAINT_DOC_PROP_H
 
-#include "dialog.h"
+#include <gcu/dialog.h>
+
+using namespace gcu;
 
 class gcpDocument;
 
-class gcpDocPropDlg: public gcpDialog
+class gcpDocPropDlg: public Dialog
 {
 public:
-	gcpDocPropDlg(gcpDocument* pDoc);
-	virtual ~gcpDocPropDlg();
+	gcpDocPropDlg (gcpDocument* pDoc);
+	virtual ~gcpDocPropDlg ();
 	
-	virtual bool Apply();
+	virtual bool Apply ();
 	
 private:
 	gcpDocument* m_pDoc;
-	GtkEntry *Title, *Name, *Mail, *CreationDate, *RevisionDate;
+	GtkEntry *Title, *Name, *Mail;
+	GtkLabel *CreationDate, *RevisionDate;
 	GtkTextView *Comments;
 	GtkTextBuffer *Buffer;
 };
