@@ -35,6 +35,7 @@
 #include <libbonobo.h>
 #include <libbonoboui.h>
 #include <gcu/document.h>
+#include <gcu/macros.h>
 #include "atom.h"
 #include "fragment.h"
 #include "bond.h"
@@ -66,6 +67,8 @@ class gcpView;
 class gcpApplication;
 class gcpDocPropDlg;
 class gcpWindow;
+class gcpTheme;
+
 class gcpDocument: public Document
 {
 	//Constructor and destructor
@@ -133,6 +136,7 @@ public:
 	void SetReadOnly (bool ro);
 	virtual double GetYAlign ();
 	gcpWindow *GetWindow () {return m_Window;}
+	void SetTheme (gcpTheme *theme);
 
 private:
 	void RemoveAtom(gcpAtom* pAtom);
@@ -158,6 +162,20 @@ private:
 	BonoboUIComponent* m_pUic;
 	gcpApplication* m_pApp;
 	gcpWindow *m_Window;
+
+/* Theme is not really a read only property, but we provide a special Set
+method */
+GCU_RO_PROP (gcpTheme*, Theme)
+GCU_PROP (double, BondLength)
+GCU_PROP (double, BondAngle)
+GCU_PROP (double, ArrowLength)
+GCU_PROP (gchar*, FontFamily)
+GCU_PROP (gchar*, TextFontFamily)
+GCU_PROP (PangoStyle, FontStyle)
+GCU_PROP (PangoWeight, FontWeight)
+GCU_PROP (PangoVariant, FontVariant)
+GCU_PROP (PangoStretch, FontStretch)
+GCU_PROP (gint, FontSize)
 };
 
 extern list<gcpDocument*> Docs;
