@@ -26,6 +26,7 @@
 #include "atom.h"
 #include "document.h"
 #include "settings.h"
+#include "theme.h"
 #include "view.h"
 #include "widgetdata.h"
 #include "libgcpcanvas/gcp-canvas-rect-ellipse.h"
@@ -106,6 +107,7 @@ void gcpElectron::SetPosition (unsigned char Pos, double angle, double distance)
 void gcpElectron::Add(GtkWidget* w)
 {
 	gcpWidgetData* pData = (gcpWidgetData*) g_object_get_data (G_OBJECT (w), "data");
+	gcpTheme* pTheme = pData->View->GetDoc ()->GetTheme ();
 	GnomeCanvasGroup* group = GNOME_CANVAS_GROUP(gnome_canvas_item_new(pData->Group, gnome_canvas_group_ext_get_type(), NULL));
 	GnomeCanvasItem* item;
 	const char *color = (m_pAtom)? ((pData->IsSelected(m_pAtom))? SelectColor: Color): "white";
@@ -114,12 +116,12 @@ void gcpElectron::Add(GtkWidget* w)
 		m_pAtom->GetCoords (&x, &y);
 		x += m_Dist * cos (angle);
 		y -= m_Dist * sin (angle);
-		x *= pData->ZoomFactor;
-		y *= pData->ZoomFactor;
+		x *= pTheme->GetZoomFactor ();
+		y *= pTheme->GetZoomFactor ();
 	} else {
 		m_pAtom->GetPosition (m_Angle, x, y);
-		x *= pData->ZoomFactor;
-		y *= pData->ZoomFactor;
+		x *= pTheme->GetZoomFactor ();
+		y *= pTheme->GetZoomFactor ();
 		x += 2. * cos (angle);
 		y -= 2. * sin (angle);
 	}
@@ -173,18 +175,19 @@ void gcpElectron::Add(GtkWidget* w)
 void gcpElectron::Update(GtkWidget* w)
 {
 	gcpWidgetData* pData = (gcpWidgetData*) g_object_get_data (G_OBJECT (w), "data");
+	gcpTheme* pTheme = pData->View->GetDoc ()->GetTheme ();
 	GnomeCanvasGroup *group = pData->Items[this];
 	double x, y, angle = m_Angle / 180. * M_PI;
 	if (m_Dist != 0.){
 		m_pAtom->GetCoords (&x, &y);
 		x += m_Dist * cos (angle);
 		y -= m_Dist * sin (angle);
-		x *= pData->ZoomFactor;
-		y *= pData->ZoomFactor;
+		x *= pTheme->GetZoomFactor ();
+		y *= pTheme->GetZoomFactor ();
 	} else {
 		m_pAtom->GetPosition (m_Angle, x, y);
-		x *= pData->ZoomFactor;
-		y *= pData->ZoomFactor;
+		x *= pTheme->GetZoomFactor ();
+		y *= pTheme->GetZoomFactor ();
 		x += 2. * cos (angle);
 		y -= 2. * sin (angle);
 	}
@@ -219,13 +222,22 @@ void gcpElectron::SetSelected(GtkWidget* w, int state)
 	GnomeCanvasGroup* group = pData->Items[this];
 	gchar *color;
 	
-	switch (state)
-	{	
-		case SelStateUnselected: color = "black"; break;
-		case SelStateSelected: color = SelectColor; break;
-		case SelStateUpdating: color = AddColor; break;
-		case SelStateErasing: color = DeleteColor; break;
-		default: color = "black"; break;
+	switch (state) {	
+	case SelStateUnselected:
+		color = "black";
+		break;
+	case SelStateSelected:
+		color = SelectColor;
+		break;
+	case SelStateUpdating:
+		color = AddColor;
+		break;
+	case SelStateErasing:
+		color = DeleteColor;
+		break;
+	default:
+		color = "black";
+		break;
 	}
 	g_object_set (G_OBJECT(g_object_get_data(G_OBJECT(group), "0")),
 				"fill_color", color, NULL);
