@@ -4,7 +4,7 @@
  * GChemPaint library
  * mesomery-arrow.cc 
  *
- * Copyright (C) 2004-2005 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -26,7 +26,9 @@
 #include "mesomery-arrow.h"
 #include "mesomery.h"
 #include "mesomer.h"
+#include "document.h"
 #include "settings.h"
+#include "theme.h"
 #include "view.h"
 #include "widgetdata.h"
 #include "libgcpcanvas/gcp-canvas-line.h"
@@ -112,24 +114,25 @@ bool gcpMesomeryArrow::Load (xmlNodePtr node)
 void gcpMesomeryArrow::Add (GtkWidget* w)
 {
 	gcpWidgetData* pData = (gcpWidgetData*) g_object_get_data (G_OBJECT (w), "data");
+	gcpTheme *pTheme = pData->View->GetDoc ()->GetTheme ();
 	GnomeCanvasPoints *points = gnome_canvas_points_new (2);
 	GnomeCanvasGroup* group = GNOME_CANVAS_GROUP(gnome_canvas_item_new (pData->Group, gnome_canvas_group_ext_get_type (), NULL));
 	GnomeCanvasItem* item;
-	points->coords[0] = m_x * pData->ZoomFactor;
-	points->coords[1] = m_y * pData->ZoomFactor;
-	points->coords[2] = (m_x + m_width) * pData->ZoomFactor;
-	points->coords[3] = (m_y + m_height) * pData->ZoomFactor;
+	points->coords[0] = m_x * pTheme->GetZoomFactor ();
+	points->coords[1] = m_y * pTheme->GetZoomFactor ();
+	points->coords[2] = (m_x + m_width) * pTheme->GetZoomFactor ();
+	points->coords[3] = (m_y + m_height) * pTheme->GetZoomFactor ();
 	item = gnome_canvas_item_new (
 								group,
 								gnome_canvas_line_ext_get_type (),
 								"points", points,
 								"fill_color", (pData->IsSelected (this))? SelectColor: Color,
-								"width_units", pData->BondWidth,
+								"width_units", pTheme->GetArrowWidth (),
 								"first_arrowhead", true,
 								"last_arrowhead", true,
-								"arrow_shape_a", pData->ArrowHeadA,
-								"arrow_shape_b", pData->ArrowHeadB,
-								"arrow_shape_c", pData->ArrowHeadC,
+								"arrow_shape_a", pTheme->GetArrowHeadA (),
+								"arrow_shape_b", pTheme->GetArrowHeadB (),
+								"arrow_shape_c", pTheme->GetArrowHeadC (),
 								"first_arrowhead_style", (unsigned char) ARROW_HEAD_BOTH,
 								"last_arrowhead_style", (unsigned char) ARROW_HEAD_BOTH,
 								NULL);
@@ -143,12 +146,13 @@ void gcpMesomeryArrow::Add (GtkWidget* w)
 void gcpMesomeryArrow::Update (GtkWidget* w)
 {
 	gcpWidgetData* pData = (gcpWidgetData*) g_object_get_data (G_OBJECT (w), "data");
+	gcpTheme *pTheme = pData->View->GetDoc ()->GetTheme ();
 	GnomeCanvasGroup* group = pData->Items[this];
 	GnomeCanvasPoints *points = gnome_canvas_points_new( 2);
-	points->coords[0] = m_x * pData->ZoomFactor;
-	points->coords[1] = m_y * pData->ZoomFactor;
-	points->coords[2] = (m_x + m_width) * pData->ZoomFactor;
-	points->coords[3] = (m_y + m_height) * pData->ZoomFactor;
+	points->coords[0] = m_x * pTheme->GetZoomFactor ();
+	points->coords[1] = m_y * pTheme->GetZoomFactor ();
+	points->coords[2] = (m_x + m_width) * pTheme->GetZoomFactor ();
+	points->coords[3] = (m_y + m_height) * pTheme->GetZoomFactor ();
 	g_object_set (G_OBJECT(g_object_get_data (G_OBJECT (group), "arrow")),
 						"points", points,
 						NULL);
