@@ -4,9 +4,7 @@
  * GChemPaint library
  * molecule.cc 
  *
- * Copyright (C) 2001-2005
- *
- * Developed by Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2001-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -62,6 +60,11 @@ static void do_show_webbook (gcpMolecule* pMol)
 	pMol->ShowWebbook ();
 }
 
+static void do_open_in_calc (gcpMolecule* pMol)
+{
+	pMol->OpenCalc ();
+}
+
 gcpMolecule::gcpMolecule(TypeId Type): Object(Type)
 {
 	m_Alignment = NULL;
@@ -107,6 +110,8 @@ void gcpMolecule::AddFragment(gcpFragment* pFragment)
 
 void gcpMolecule::AddBond(gcpBond* pBond)
 {
+	if (pBond->GetAtom (0) && pBond->GetAtom (1))
+		CheckCrossings (pBond);
 	m_Bonds.push_back(pBond);
 	AddChild(pBond);
 	EmitSignal (OnChangedSignal);
@@ -178,7 +183,7 @@ bool gcpMolecule::Merge(gcpMolecule* pMolecule, bool RemoveDuplicates)
 				ma->pAtom = pAtom;
 				endj = AtomMap.end ();
 				for (j = AtomMap.begin (); j != endj; j++) {
-					if (b1 = (gcpBond*) pAtom->GetBond ((*j).second->pAtom)) {
+					if ((b1 = (gcpBond*) pAtom->GetBond ((*j).second->pAtom))) {
 						b0 = (gcpBond*) (*i)->GetBond ((*j).first);
 						if (b0) {
 							ma->sb++;
@@ -382,6 +387,7 @@ bool gcpMolecule::Load (xmlNodePtr node)
 		if (pDoc)
 			pDoc->AddBond ((gcpBond*) pObject);
 		child = GetNextNodeByName (child->next, "bond");
+		CheckCrossings ((gcpBond*) pObject);
 	}
 	if (!m_Atoms.empty ()) {
 		gcpAtom* pAtom = m_Atoms.front ();
@@ -457,7 +463,7 @@ Object* gcpMolecule::GetAtomAt(double x, double y, double z)
 	std::list<gcpBond*>::iterator n, end = m_Bonds.end();
 	Object* pObj = NULL;
 	for (n = m_Bonds.begin(); n != end; n++)
-		if (pObj = (*n)->GetAtomAt (x, y))
+		if ((pObj = (*n)->GetAtomAt (x, y)))
 			break;
 	return pObj;
 }
@@ -514,6 +520,10 @@ bool gcpMolecule::BuildContextualMenu (GtkUIManager *UIManager, Object *object)
 		g_signal_connect_swapped (action, "activate", G_CALLBACK (do_build_smiles), this);
 		gtk_action_group_add_action (group, action);
 		gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Molecule'><menuitem action='smiles'/></menu></popup></ui>", -1, NULL);
+		action = gtk_action_new ("calc", _("Open in Calculator"), NULL, NULL);
+		g_signal_connect_swapped (action, "activate", G_CALLBACK (do_open_in_calc), this);
+		gtk_action_group_add_action (group, action);
+		gtk_ui_manager_add_ui_from_string (UIManager, "<ui><popup><menu action='Molecule'><menuitem action='calc'/></menu></popup></ui>", -1, NULL);
 		result = true;
 	}
 	if (m_Bonds.size ()) {
@@ -633,11 +643,67 @@ void gcpMolecule::BuildOBMol (OBMol &Mol)
 	Mol.EndModify();
 }
 
+void gcpMolecule::BuildOBMol2D (OBMol &Mol)
+{
+	double xav = 0., yav = 0.;
+	unsigned n = m_Atoms.size();
+	map<string, unsigned> AtomTable;
+	list<gcpAtom*>::iterator ia, enda = m_Atoms.end ();
+	list<gcpBond*> BondList;
+	double x, y, z;
+	for (ia = m_Atoms.begin(); ia != enda; ia++) {
+		(*ia)->GetCoords(&x, &y, &z);
+		xav += x;
+		yav += y;
+	}
+	xav /= n;
+	yav /= n;
+	gcpAtom* pgAtom;
+	OBAtom obAtom;
+	unsigned index = 1;
+	map<Atom*, Bond*>::iterator i;
+	Mol.BeginModify();
+	Mol.ReserveAtoms(n);
+	Mol.SetDimension (2);
+	for (ia = m_Atoms.begin(); ia != enda; ia++) {
+		pgAtom = *ia;
+		AtomTable [pgAtom->GetId ()] = index;
+		obAtom.SetIdx(index++);
+		obAtom.SetAtomicNum(pgAtom->GetZ());
+		pgAtom->GetCoords(&x, &y, &z);
+		// Scans the atom bonds and change z to try conservation of stereochemistry
+		obAtom.SetVector((xav - x )/ 100, (yav - y) / 100, 0.);
+		Mol.AddAtom(obAtom);
+		obAtom.Clear();
+	}
+	list<gcpBond*>::iterator j, endb = m_Bonds.end ();
+	int start, end, order, flag;
+	for (j = m_Bonds.begin(); j != endb; j++)
+	{
+		order = (*j)->GetOrder();
+		start = AtomTable[(*j)->GetAtom(0)->GetId()];
+		end = AtomTable[(*j)->GetAtom(1)->GetId()];
+		switch((*j)->GetType())
+		{
+			case UpBondType:
+				flag = OB_WEDGE_BOND;
+				break;
+			case DownBondType:
+				flag = OB_HASH_BOND;
+				break;
+			default:
+				flag = 0;
+		}
+		Mol.AddBond(start, end, order, flag);
+	}
+	Mol.EndModify();
+}
+
 void gcpMolecule::BuildInChI ()
 {
 	OBMol Mol;
 	OBConversion Conv;
-	BuildOBMol (Mol);
+	BuildOBMol2D (Mol);
 	OBFormat *pInChIFormat = Conv.FindFormat ("inchi"), *pMolFormat = Conv.FindFormat ("mol");
 	if (pInChIFormat) {
 		Conv.SetInAndOutFormats (pMolFormat, pInChIFormat);
@@ -688,7 +754,7 @@ void gcpMolecule::BuildSmiles ()
 	OBConversion Conv;
 	OBFormat* pOutFormat = Conv.FindFormat ("smi");
 	Conv.SetInAndOutFormats (pOutFormat, pOutFormat);
-	BuildOBMol (Mol);
+	BuildOBMol2D (Mol);
 	ostringstream ofs;
 	char *old_num_locale = g_strdup (setlocale (LC_NUMERIC, NULL));
 	setlocale (LC_NUMERIC, "C");
@@ -713,6 +779,9 @@ void gcpMolecule::ShowWebbook ()
 		BuildInChI ();
 	if (m_InChI.length () == 0)
 		return; //should emit at least a warning
+	string::size_type t;
+	while ((t = m_InChI.find ('+')) != string::npos)
+		m_InChI.replace (t, 1, "%2b");
 	string uri = string ("http://webbook.nist.gov/cgi/cbook.cgi?Name=") + m_InChI.c_str () + string ("&Units=SI");
 	((gcpDocument*) GetDocument ())->GetApplication ()->ShowURI (uri);
 }
@@ -722,3 +791,31 @@ bool gcpMolecule::OnSignal (SignalId Signal, Object *Child)
 	m_Changed = true;
 	return true;
 }
+
+void gcpMolecule::OpenCalc ()
+{
+	list<gcpAtom*>::iterator ia, enda = m_Atoms.end ();
+	ostringstream ofs;
+	int nH;
+#warning "the following line should be edited for stable releases"
+	ofs << "gchemcalc-unstable ";
+	for (ia = m_Atoms.begin(); ia != enda; ia++) {
+		ofs << (*ia)->GetSymbol();
+		nH = (*ia)->GetAttachedHydrogens ();
+		if (nH > 0) {
+			ofs << "H" << nH;
+		}
+	}
+	g_spawn_command_line_async (ofs.str ().c_str (), NULL);
+}
+
+void gcpMolecule::CheckCrossings (gcpBond *pBond)
+{
+	gcpView *pView = reinterpret_cast<gcpDocument*> (GetDocument ())->GetView ();
+	list<gcpBond*>::iterator i, iend = m_Bonds.end ();
+	for (i = m_Bonds.begin (); i != iend; i++)
+		if (((*i) != pBond) && (*i)->IsCrossing (pBond)) {
+			pView->Update (pBond);
+			pView->Update (*i);
+		}
+}
