@@ -28,6 +28,7 @@
 #include "cycle.h"
 #include "fragment.h"
 #include <list>
+#include <openbabel/mol.h>
 
 class gcpMolecule: public Object
 {
@@ -50,9 +51,18 @@ public:
 	virtual Object* GetAtomAt(double x, double y, double z = 0.);
 	virtual double GetYAlign ();
 	virtual bool BuildContextualMenu (GtkUIManager *UIManager, Object *object);
+	virtual bool OnSignal (SignalId Signal, Object *Child);
 	void ExportToGhemical ();
 	void SelectAlignmentItem (Object *child);
 	string GetAlignmentId () {return (m_Alignment)? m_Alignment->GetId (): "";}
+	void BuildOBMol (OBMol &Mol);
+	void BuildOBMol2D (OBMol &Mol);
+	void ShowInChI ();
+	void BuildInChI ();
+	void BuildSmiles ();
+	void ShowWebbook ();
+	void OpenCalc ();
+	void CheckCrossings (gcpBond *pBond);
 
 private:
 	list<gcpCycle*> m_Cycles;
@@ -61,6 +71,8 @@ private:
 	list<gcpFragment*> m_Fragments;
 	list<gcpBond*> m_Bonds;
 	Object *m_Alignment;
+	string m_InChI;
+	bool m_Changed;
 };
 
 #endif // GCHEMPAINT_MOLECULE_H
