@@ -41,12 +41,13 @@ gcpPlugin::~gcpPlugin ()
 
 void gcpPlugin::LoadPlugins ()
 {
-	GDir* dir = g_dir_open (PLUGINSDIR, 0, NULL);
+	GDir* dir = g_dir_open (LIBDIR"/gchempaint-unstable/plugins", 0, NULL);
 	if (dir ==  NULL) return;
 	const char* name;
 	while ((name = g_dir_read_name (dir))) {
-		if (strcmp (name + strlen (name) - 3, ".so")) continue;
-		name = g_strconcat (PLUGINSDIR"/", name, NULL);
+		if ((strcmp (name + strlen (name) - 3, ".so")))
+			continue;
+		name = g_strconcat (LIBDIR"/gchempaint-unstable/plugins/", name, NULL);
 		if (!dlopen (name, RTLD_NOW)) puts (dlerror());
 		g_free ((void*) name);
 	}
