@@ -4,9 +4,7 @@
  * GChemPaint library
  * reaction.h 
  *
- * Copyright (C) 2002-2004
- *
- * Developed by Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2004 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -20,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -41,8 +39,10 @@ public:
 	virtual void Add ( GtkWidget* w);
 	virtual bool Build (list<Object*>& Children) throw (invalid_argument);
 	virtual void Transform2D (Matrix2D& m, double x, double y);
+	virtual bool BuildContextualMenu (GtkUIManager *UIManager, Object *object);
 	virtual bool OnSignal (SignalId Signal, Object *Child);
 	virtual bool Load (xmlNodePtr);
+	virtual double GetYAlign ();
 };
 
 #endif	//GCHEMPAINT_REACTION_H
