@@ -4,9 +4,7 @@
  * GChemPaint library
  * text.cc 
  *
- * Copyright (C) 2002-2005
- *
- * Developed by Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2002-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -31,6 +29,7 @@
 #include "document.h"
 #include  "application.h"
 #include "settings.h"
+#include "theme.h"
 #include "libgcpcanvas/gcp-canvas-group.h"
 #include "libgcpcanvas/gcp-canvas-rect-ellipse.h"
 #include "libgcpcanvas/gprintable.h"
@@ -215,14 +214,13 @@ bool gcpText::SaveNode(xmlDocPtr xml, xmlNodePtr node, GtkTextIter *iter, GtkTex
 			*iter = *end;
 		}
 		char *rn, *text = gtk_text_buffer_get_text(m_buf, &start, iter, true);
-		while (rn = strchr(text, '\n'))
-		{
-			xmlNodeAddContentLen(node, (xmlChar*)text, rn - text);
+		while ((rn = strchr (text, '\n'))) {
+			xmlNodeAddContentLen (node, (xmlChar*) text, rn - text);
 			text = rn + 1;
-			xmlNodePtr child = xmlNewDocNode(xml, NULL, (xmlChar*)"br", NULL);
-			xmlAddChild(node, child);
+			xmlNodePtr child = xmlNewDocNode (xml, NULL, (xmlChar*) "br", NULL);
+			xmlAddChild (node, child);
 		}
-		xmlNodeAddContent(node, (xmlChar*)text);
+		xmlNodeAddContent (node, (xmlChar*) text);
 	}
 	else
 	{
@@ -387,6 +385,7 @@ bool gcpText::LoadTaggedNode (xmlNodePtr node, GtkTextIter* CurIter, const char*
 void gcpText::Add(GtkWidget* w)
 {
 	gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(w), "data");
+	gcpTheme *pTheme = pData->View->GetDoc ()->GetTheme ();
 	if (m_ascent <= 0)
 	{
 		PangoContext* pc = pData->View->GetPangoContext();
@@ -401,10 +400,10 @@ void gcpText::Add(GtkWidget* w)
 	GnomeCanvasItem* item = gnome_canvas_item_new(
 						group,
 						gnome_canvas_rect_ext_get_type(),
-						"x1", m_x * pData->ZoomFactor - pData->Padding,
-						"y1", m_y * pData->ZoomFactor - pData->Padding - m_ascent,
-						"x2", m_x * pData->ZoomFactor + m_length + pData->Padding,
-						"y2", m_y * pData->ZoomFactor + m_height + pData->Padding - m_ascent,
+						"x1", m_x * pTheme->GetZoomFactor () - pTheme->GetPadding (),
+						"y1", m_y * pTheme->GetZoomFactor () - pTheme->GetPadding () - m_ascent,
+						"x2", m_x * pTheme->GetZoomFactor () + m_length + pTheme->GetPadding (),
+						"y2", m_y * pTheme->GetZoomFactor () + m_height + pTheme->GetPadding () - m_ascent,
 						"fill_color", "white",
 						"outline_color", "white",
 						NULL);
@@ -415,8 +414,8 @@ void gcpText::Add(GtkWidget* w)
 						group,
 						gnome_canvas_rich_text_gcp_get_type(),
 						"text", "",
-						"x", m_x * pData->ZoomFactor,
-						"y", m_y * pData->ZoomFactor - m_ascent,
+						"x", m_x * pTheme->GetZoomFactor (),
+						"y", m_y * pTheme->GetZoomFactor () - m_ascent,
 						"width", m_length,
 						"height", m_height,
 						"grow_height", false,
@@ -637,18 +636,19 @@ bool gcpText::OnEndUserAction(GtkTextBuffer *textbuffer)
 void gcpText::Update(GtkWidget* w)
 {
 	gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(w), "data");
+	gcpTheme *pTheme = pData->View->GetDoc ()->GetTheme ();
 	GnomeCanvasGroup* group = pData->Items[this];
 	g_object_set(G_OBJECT(g_object_get_data(G_OBJECT(group), "text")),
-						"x", m_x * pData->ZoomFactor,
-						"y", m_y * pData->ZoomFactor - m_ascent,
+						"x", m_x * pTheme->GetZoomFactor (),
+						"y", m_y * pTheme->GetZoomFactor () - m_ascent,
 						"width", m_length,
 						"height", m_height,
 						NULL);
 	g_object_set(G_OBJECT(g_object_get_data(G_OBJECT(group), "rect")),
-						"x1", m_x * pData->ZoomFactor - pData->Padding,
-						"y1", m_y * pData->ZoomFactor - pData->Padding - m_ascent,
-						"x2", m_x * pData->ZoomFactor + m_length + pData->Padding,
-						"y2", m_y * pData->ZoomFactor + m_height + pData->Padding - m_ascent,
+						"x1", m_x * pTheme->GetZoomFactor () - pTheme->GetPadding (),
+						"y1", m_y * pTheme->GetZoomFactor () - pTheme->GetPadding () - m_ascent,
+						"x2", m_x * pTheme->GetZoomFactor () + m_length + pTheme->GetPadding (),
+						"y2", m_y * pTheme->GetZoomFactor () + m_height + pTheme->GetPadding () - m_ascent,
 						NULL);
 }
 
