@@ -29,6 +29,8 @@
 #include "selectiontool.h"
 #include "erasertool.h"
 #include "group.h"
+#include "gcp-stock-pixbufs.h"
+#include <glib/gi18n-lib.h>
 
 gcpSelectionPlugin plugin;
 
@@ -46,9 +48,42 @@ gcpSelectionPlugin::~gcpSelectionPlugin()
 {
 }
 
+static gcpIconDesc icon_descs[] = {
+	{"gcp_Selection", gcp_selection_24},
+	{"gcp_Eraser", gcp_eraser_24},
+	{"gcp_Horiz", gcp_horiz_24},
+	{"gcp_Vert", gcp_vert_24},
+	{"gcp_Rotate", gcp_rotate_24},
+	{"gcp_Merge", gcp_merge_24},
+	{NULL, NULL},
+};
+
+static GtkRadioActionEntry entries[] = {
+	{	"Select", "gcp_Selection", N_("Select"), NULL,
+		N_("Select one or more objects"),
+		0	},
+	{	"Erase", "gcp_Eraser", N_("Erase"), NULL,
+		N_("Eraser"),
+		0	},
+};
+
+static const char *ui_description =
+"<ui>"
+"  <toolbar name='SelectToolbar'>"
+"	 <placeholder name='Select1'>"
+"      <toolitem action='Select'/>"
+"      <toolitem action='Erase'/>"
+"	 </placeholder>"
+"	 <placeholder name='Select2'/>"
+"	 <placeholder name='Select3'/>"
+"  </toolbar>"
+"</ui>";
+
 void gcpSelectionPlugin::Populate (gcpApplication* App)
 {
-	App->AddUI (GLADEDIR"/selection.xml");
+	App->AddUI (DATADIR"/gchempaint-unstable/ui/selection.xml");
+	App->AddActions (entries, G_N_ELEMENTS (entries), ui_description, icon_descs);
+	App->RegisterToolbar ("SelectToolbar", 0);
 	new gcpSelectionTool (App);
 	new gcpEraserTool (App);
 	App->ActivateTool ("Select", true);
