@@ -27,6 +27,7 @@
 #include "lib/application.h"
 #include "templatetool.h"
 #include "templatetree.h"
+#include "category.h"
 #include <glib/gi18n-lib.h>
 #include <sys/stat.h>
 
@@ -44,7 +45,7 @@ gcpTemplatesPlugin::gcpTemplatesPlugin(): gcpPlugin()
 	xmlIndentTreeOutput = true;
 	xmlKeepBlanksDefault (0);
 	if (dir) {
-		while (name = g_dir_read_name (dir)) {
+		while ((name = g_dir_read_name (dir))) {
 			if (strcmp (name + strlen (name) - 4, ".xml")) continue;
 			name = g_strconcat (DATADIR"/gchempaint/templates/", name, NULL);
 			doc = xmlParseFile (name);
@@ -59,7 +60,7 @@ gcpTemplatesPlugin::gcpTemplatesPlugin(): gcpPlugin()
 	dir = g_dir_open (path, 0, NULL);
 	if (dir) {
 		bool is_user_templates;
-		while (name = g_dir_read_name (dir)) {
+		while ((name = g_dir_read_name (dir))) {
 			if (strcmp (name + strlen (name) - 4, ".xml")) continue;
 			is_user_templates = !strcmp (name, "templates.xml");
 			name = g_strconcat (path, "/", name, NULL);
@@ -128,6 +129,7 @@ void gcpTemplatesPlugin::ParseNodes (xmlNodePtr node, bool writeable)
 {
 	xmlNodePtr next, child, next_child;
 	gcpTemplate* t;
+	gcpTemplateCategory *c;
 	bool lang_matched, cat_matched;
 	xmlChar *category, *name;
 	string key;
@@ -218,10 +220,16 @@ void gcpTemplatesPlugin::ParseNodes (xmlNodePtr node, bool writeable)
 			}
 			if (name) xmlFree (name);
 			if (category) {
-				set<string>::iterator it = categories.find ((char*) category);
-				if (it == categories.end()) categories.insert ((char*) category);
+				c = TemplateCategories[(char*) category];
+				if (c == NULL)
+					c = new gcpTemplateCategory ((char*) category);
 				xmlFree (category);
+			} else {
+				c = TemplateCategories[_("Miscellaneous")];
+				if (c == NULL)
+					c = new gcpTemplateCategory (_("Miscellaneous"));
 			}
+			c->AddTemplate (t);
 		}
 		node = next;
 	}
