@@ -4,7 +4,7 @@
  * GChemPaint templates plugin
  * templatetool.cc 
  *
- * Copyright (C) 2004-2005 Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -22,9 +22,11 @@
  * USA
  */
 
+#include "gchempaint-config.h"
 #include "templatetool.h"
 #include "templatetree.h"
 #include "lib/document.h"
+#include "lib/theme.h"
 #include <gtk/gtk.h>
 
 xmlDocPtr xml;
@@ -153,9 +155,122 @@ enum {
 	NUM_COLUMNS
 };
 
-static void on_changed(GtkTreeSelection *Selection, gcpTemplateToolDlg* dlg)
+static void on_template_changed(GtkComboBox *combo, gcpTemplateTool *tool)
 {
-	dlg->OnChanged(Selection);
+	tool->OnChanged (combo);
+}
+
+static void on_size (GtkWidget *widget, GtkAllocation *allocation, gcpTemplateTool *tool)
+{
+	tool->OnPreviewSize (allocation);
+}
+
+GtkWidget *gcpTemplateTool::GetPropertyPage ()
+{
+	GladeXML *xml = glade_xml_new (DATADIR"/gchempaint-unstable/ui/templates.glade", "templates", GETTEXT_PACKAGE);
+	gcpTemplateTree *tree = (gcpTemplateTree*) m_pApp->GetTool ("TemplateTree");
+	if (!tree)
+		return NULL;
+	GtkComboBox *combo = GTK_COMBO_BOX (glade_xml_get_widget (xml, "templates-combo"));
+	gtk_combo_box_set_model (combo, tree->GetModel ());
+	GtkCellRenderer* renderer = (GtkCellRenderer*) gtk_cell_renderer_text_new ();
+	g_object_set (renderer, "xalign", 0.0, NULL);
+	gtk_cell_layout_clear (GTK_CELL_LAYOUT (combo));
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo), renderer, true);
+	gtk_cell_layout_add_attribute (GTK_CELL_LAYOUT (combo),
+								renderer, "text",
+								NAME_COLUMN);
+	gtk_combo_box_set_active (combo, 0);
+	g_signal_connect (G_OBJECT (combo), "changed", G_CALLBACK (on_template_changed), this);
+	m_DeleteBtn = glade_xml_get_widget (xml, "delete");
+	m_Book = GTK_NOTEBOOK (glade_xml_get_widget (xml, "book"));
+	g_signal_connect (m_Book, "size-allocate", G_CALLBACK (on_size), this);
+	return glade_xml_get_widget (xml, "templates");
+}
+
+void gcpTemplateTool::OnChanged (GtkComboBox *combo)
+{
+	gcpTemplateTree *tree = (gcpTemplateTree*) m_pApp->GetTool ("TemplateTree");
+	if (!tree)
+		return;
+	GtkTreeIter iter;
+	if (gtk_combo_box_get_active_iter (combo, &iter)) {
+		GtkTreePath *path = gtk_tree_model_get_path ((GtkTreeModel*) tree->GetModel (), &iter);
+		char* path_string = gtk_tree_path_to_string (path);
+		m_Template = tree->GetTemplate (path_string);
+		int page;
+		if (m_Template) {
+			if (!m_Template->doc) {
+				m_Template->doc = new gcpDocument (NULL, false);
+				gcpTheme *pTheme = m_Template->doc->GetTheme ();
+				m_Template->doc->SetEditable (false);
+				m_Template->data =  (gcpWidgetData*) g_object_get_data (G_OBJECT (m_Template->doc->GetView ()->CreateNewWidget ()), "data");
+				m_Template->doc->AddData (m_Template->node);
+				m_Template->data->UnselectAll ();
+				gnome_canvas_update_now (GNOME_CANVAS (m_Template->doc->GetWidget ()));
+				m_Template->data->GetObjectBounds (m_Template->doc, &m_Template->rect);
+				m_Template->doc->Move (-m_Template->rect.x0 / pTheme->GetZoomFactor (), -m_Template->rect.y0 / pTheme->GetZoomFactor ());
+				page = -1;
+			} else
+				page = gtk_notebook_page_num (m_Book, m_Template->w);
+			if (page < 0) {
+				m_Template->data =  (gcpWidgetData*) g_object_get_data (G_OBJECT (m_Template->doc->GetView ()->CreateNewWidget ()), "data");
+				gnome_canvas_update_now (GNOME_CANVAS (m_Template->doc->GetWidget ()));
+				m_Template->data->GetObjectBounds (m_Template->doc, &m_Template->rect);
+				m_Template->w = gtk_scrolled_window_new (NULL, NULL);
+/*				GtkWidget *wd = gtk_notebook_get_nth_page (m_Book, 0);
+				double w = (wd->allocation.width - 2 * m_Template->w->style->xthickness) / (m_Template->rect.x1 - m_Template->rect.x0);
+				double h = (wd->allocation.height - 2 * m_Template->w->style->ythickness) / (m_Template->rect.y1 - m_Template->rect.y0);*/
+				gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (m_Template->w), GTK_SHADOW_NONE);
+				gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (m_Template->w), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
+				gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW (m_Template->w), m_Template->doc->GetWidget ());
+/*				if (w < 1. || h < 1.) {
+					pData->Zoom = MIN (w, h);
+					gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (m_Template->doc->GetWidget ()), pData->Zoom);
+				}*/
+				gtk_widget_show_all (m_Template->w);
+				gtk_notebook_append_page (m_Book, m_Template->w, NULL);
+				page = gtk_notebook_page_num (m_Book, m_Template->w);
+			}
+			GtkWidget *wd = gtk_notebook_get_nth_page (m_Book, 0);
+			// Why 4 and not 2 in following lines?
+			double w = (wd->allocation.width - 4 * m_Template->w->style->xthickness) / (m_Template->rect.x1 - m_Template->rect.x0);
+			double h = (wd->allocation.height - 4 * m_Template->w->style->ythickness) / (m_Template->rect.y1 - m_Template->rect.y0);
+			if (w < 1. || h < 1.) {
+				m_Template->data->Zoom = MIN (w, h);
+				gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (m_Template->doc->GetWidget ()), m_Template->data->Zoom);
+			}
+			gtk_notebook_set_current_page (m_Book, page);
+			gtk_widget_set_sensitive (m_DeleteBtn, m_Template->writeable);
+		} else {
+			// a category has been selected
+			m_Template = NULL;
+			GtkTreeModel *model = tree->GetModel ();
+			char const *name;
+			gtk_tree_model_get (model, &iter, 0, &name, NULL);
+puts(name);
+			gtk_widget_set_sensitive (m_DeleteBtn, false);
+			gtk_notebook_set_current_page (m_Book, 0);
+		}
+		g_free (path_string);
+		gtk_tree_path_free (path);		
+	}
+}
+
+void gcpTemplateTool::OnPreviewSize (GtkAllocation *allocation)
+{
+	if (m_Template) {
+		double w = (allocation->width - 4 * m_Template->w->style->xthickness) / (m_Template->rect.x1 - m_Template->rect.x0);
+		double h = (allocation->height - 4 * m_Template->w->style->ythickness) / (m_Template->rect.y1 - m_Template->rect.y0);
+		m_Template->data->Zoom = (w < 1. || h < 1.)? MIN (w, h): 1.;
+		gnome_canvas_set_pixels_per_unit (GNOME_CANVAS (m_Template->data->Canvas), m_Template->data->Zoom);
+		g_signal_emit_by_name (m_Template->data->Canvas, "update_bounds");
+	}
+}
+
+static void on_changed (GtkTreeSelection *Selection, gcpTemplateToolDlg* dlg)
+{
+	dlg->OnChanged (Selection);
 }
 
 static void on_add_template (GtkWidget *w, gcpTemplateToolDlg *dlg)
@@ -168,10 +283,13 @@ static void on_delete_template (GtkWidget *w, gcpTemplateToolDlg *dlg)
 	dlg->OnDeleteTemplate ();
 }
 
-gcpTemplateToolDlg::gcpTemplateToolDlg(gcpApplication* App, gcpTemplate *t): Dialog(App, DATADIR"/gchempaint-unstable/ui/templates.glade", "templates")
+gcpTemplateToolDlg::gcpTemplateToolDlg(gcpApplication* App, gcpTemplate *t): Dialog(App, DATADIR"/gchempaint-unstable/ui/templates-old.glade", "templates")
 {
 	tree = (gcpTemplateTree*) App->GetTool ("TemplateTree");
-	if (!tree) {delete this; return;}
+	if (!tree) {
+		delete this;
+		return;
+	}
 	GtkTreeView* view = (GtkTreeView*)glade_xml_get_widget(xml, "template_tree_view");
 	gtk_tree_view_set_model (view, model = tree->GetModel());
 	GtkTreeSelection* Selection = gtk_tree_view_get_selection(view);
@@ -184,7 +302,7 @@ gcpTemplateToolDlg::gcpTemplateToolDlg(gcpApplication* App, gcpTemplate *t): Dia
 								NULL);
 	g_signal_connect (G_OBJECT(Selection), "changed", G_CALLBACK (on_changed), this);
 	temp = t;
-	pDoc = new gcpDocument(reinterpret_cast<gcpApplication*> (m_App), true);
+	pDoc = new gcpDocument (reinterpret_cast<gcpApplication*> (m_App), true);
 	pDoc->SetEditable (false);
 	GtkWidget* w;
 	GtkScrolledWindow* scroll = (GtkScrolledWindow*)glade_xml_get_widget(xml, "scrolledcanvas");
@@ -228,7 +346,7 @@ bool gcpTemplateToolDlg::Apply()
 	return true;
 }
 
-void gcpTemplateToolDlg::OnChanged(GtkTreeSelection *Selection)
+void gcpTemplateToolDlg::OnChanged (GtkTreeSelection *Selection)
 {
 	GtkTreeIter iter;
 	GtkTreePath *path;
@@ -241,7 +359,7 @@ void gcpTemplateToolDlg::OnChanged(GtkTreeSelection *Selection)
 		gtk_tree_path_free (path);
 		map<string, Object*>::iterator it;
 		Object* obj;
-		while (obj = pDoc->GetFirstChild(it)) {
+		while ((obj = pDoc->GetFirstChild(it))) {
 			pDoc->Remove (obj);
 			pDoc->PopOperation ();
 		}
@@ -374,7 +492,7 @@ bool gcpNewTemplateToolDlg::Apply()
 	temp->category = category;
 	if (!user_templates) {
 		user_templates = xmlNewDoc ((xmlChar*) "1.0");
-		user_templates->children =  xmlNewDocNode(user_templates, NULL, (xmlChar*) "templates", NULL);
+		xmlDocSetRootElement (user_templates,  xmlNewDocNode(user_templates, NULL, (xmlChar*) "templates", NULL));
 		char* filename = g_strconcat (getenv ("HOME"), "/.gchempaint/templates/templates.xml", NULL);
 		user_templates->URL = xmlStrdup ((xmlChar*) filename);
 		g_free (filename);
