@@ -4,9 +4,7 @@
  * GChemPaint templates plugin
  * templatetree.cc 
  *
- * Copyright (C) 2004-2005
- *
- * Developed by Jean Bréfort <jean.brefort@normalesup.org>
+ * Copyright (C) 2004-2006 Jean Bréfort <jean.brefort@normalesup.org>
  *
  * This program is free software; you can redistribute it and/or 
  * modify it under the terms of the GNU General Public License as 
@@ -24,9 +22,11 @@
  * USA
  */
 
+#include "gchempaint-config.h"
 #include "templatetree.h"
 #include "templatetool.h"
 #include "gtkcombotoolitem.h"
+#include <lib/document.h>
 #include <glib/gi18n.h>
 
 static void on_changed (GtkComboBox *combo, gcpApplication *App)
@@ -60,17 +60,19 @@ static void on_templates(GtkWidget *w, gcpApplication* App)
 }
 
 map<string, gcpTemplate*> Templates, TempbyName;
-set<string> categories;
 set<xmlDocPtr> docs;
 xmlDocPtr user_templates = NULL;
 
 gcpTemplate::gcpTemplate ()
 {
 	node = NULL;
+	doc = NULL;
 }
 
 gcpTemplate::~gcpTemplate()
 {
+	if (doc)
+		delete (doc);
 }
 
 enum {
